#! /usr/bin/perl
#
# $Id: elagi.pl,v 1.1 2004/04/03 23:34:25 jhayes Exp $ */
# Copyright  2004 The Regents of the University of California. 
# All Rights Reserved. 
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for educational, research and non-profit purposes, without
# fee, and without a written agreement is hereby granted, provided that the
# above copyright notice, this paragraph and the following three paragraphs
# appear in all copies. 
#
# Permission to incorporate this software into commercial products may be
# obtained by contacting
# Technology Transfer Office 
# 9500 Gilman Drive 
# 411 University Center 
# University of California 
# La Jolla, CA 92093-0093
# (858) 534-5815
# invent@ucsd.edu
#
# This software program and documentation are copyrighted by The Regents of
# the University of California. The software program and documentation are
# supplied "as is", without any accompanying services from The Regents. The
# Regents does not warrant that the operation of the program will be
# uninterrupted or error-free. The end-user understands that the program was
# developed for research purposes and is advised not to rely exclusively on
# the program for any reason. 
#
# IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
# DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
# LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
# EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
# WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
# HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
# OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
# MODIFICATIONS. 


use PolyDoc;

# Writes #html# packaged into an HTML page with a common appearance to #file#,
# using #title# as the page title.
sub ElagiPage {
  my ($file, $title, $html) = @_;
  print "Producing $file\n";
  FileWrite($file,
    HtmlHeader
      ($title,
       '  <link rel="stylesheet" href="general.css" type="text/css">' . "\n" .
       HtmlStyle('span.code', 'aqua;bold',
                 'span.doc',  'margin-left:5%;margin-bottom:0.25in',
                 'span.kept', 'margin-bottom:0.5in') . "\n") .
    HtmlBody(
      '<p align="center"> <img src="elagi.gif" alt="elagi">' . "\n\n" .
      '<h1 class="title">' . $title . "</h1>\n\n" .
      AddLinks(ReplaceAll($html, '#(\w+)#', '<i>$1</i>'))
    )
  );
}

my %html;
my %titles = ('bulkcopy'  => 'Multi-File Copy',
              'elbase'    => 'Spawn Processes/Copy Files',
              'elmetric'  => 'Obtain Grid Metrics',
              'gmingrp'   => 'JIT Ganglia System',
              'nwsclient' => 'Obtain NWS Metrics',
              'rashgrp'   => 'Reverse Access Shell System');

foreach $title (keys %titles) {
  Extract(FileRead("../$title.h"), 'c', "$title.html", 'This package');
  $html{$title} = ExtractedHtml("$title.html");
  while($html{$title} =~ s/(:\n<p>)([^<]+)/$1<pre>$2<\/pre>/) { }
}

foreach $title (keys %html) {
  ElagiPage("$title.html", $titles{$title}, $html{$title});
}

ElagiPage('bugs.html', 'Bugs and Limitations', FileRead('bugs'));
ElagiPage('download.html', 'Download', FileRead('download'));
ElagiPage('install.html', 'Installation and Use', FileRead('install'));
ElagiPage('introduction.html', 'Introduction', FileRead('introduction'));
ElagiPage('release.html', 'Release Notes', FileRead('release'));
print "Producing menu.html\n";
FileWrite('menu.html',
          HtmlHeader
            ("Menu",
             '<base target=mainFrame>' .
             '<link rel="stylesheet" href="general.css" type="text/css">') .
          HtmlBody
            (HtmlTable(1, '<a href=introduction.html>Introduction</a>',
                          '<a href=download.html>Download</a>',
                          '<a href=install.html>Installation</a>',
                          '<a href=release.html>Release Notes</a>',
                          '<a href=bugs.html>Bugs</a>',
                          '<a href=bulkcopy.html>bulkcopy</a>',
                          '<a href=elbase.html>elbase</a>',
                          '<a href=elmetric.html>elmetric</a>',
                          '<a href=gmingrp.html>gmingrp</a>',
                          '<a href=nwsclient.html>nwsclient</a>',
                          '<a href=rashgrp.html>rashgrp</a>',
                          '<a href=nameindex.html>Declared Names</a>'))
);
@names = DeclaredNames();
ElagiPage('nameindex.html', 'Declared Names', HtmlTable(1, sort @names));
print "Producing index.html\n";
FileWrite('index.html',
          HtmlHeader('Elagi') .
          '<frameset cols="200,*">' . "\n" .
          '  <frame src="menu.html" name=menuFrame>' . "\n" .
          '  <frame src="introduction.html" name=mainFrame>' . "\n" .
          "</frameset>\n" .
          "<noframes>\n" .
          "  Sorry, this page requires frames\n" .
          "</html>");
