/* $Id: ecp.c,v 1.9 2004/03/15 16:55:10 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This program provides a command-line interface to Elagi's file copy
  * services.  Usage:
  *
  *   ecp source dest
  *
  * where both #source# and #dest# have the format [[type#]server#]path.
  * If no server is provided for a file it defaults to a local file.  If a
  * server but no type is provided, the type defaults to ssh for the source and
  * to the source type for the destination.
  */

#include <stdio.h>  /* fprintf */
#include <stdlib.h> /* free */
#include <string.h> /* strchr strcmp strdup */
#define ASENV_SHORT_NAMES
#include <envseed.h>
#include <oshseed.h>
#include "elbase.h"

int
main(int argc,
     char **argv) {

  const char *VALID =
    "--scriptgen yes|no\n"
    " string  [[type#]server#]path [[type#]server#]path";
  const char *SERVER_TYPE_NAMES[ELBASE_STORAGE_SERVICE_COUNT + 1] = {
    "cp", "ftp", "gass", "scp", "sftp", "srb", NULL
  };

  char *c;
  char *dest;
  char *destPath;
  char *destServer;
  ELBASE_StorageServiceTypes destType;
  ParsedArgv pa;
  ELBASE_Process pid;
  int scriptgen;
  char *source;
  char *sourcePath;
  char *sourceServer;
  ELBASE_StorageServiceTypes sourceType;
  int status;

  if((pa = ParseArgv((const char **)argv, VALID, NULL)) == NULL ||
     SwitchValueN(pa, "", 2, NULL) == NULL) {
    fprintf(stderr, "usage: ecp\n%s\n", VALID);
    return 1;
  }

  source = strdup(SwitchValueN(pa, "", 1, NULL));
  if((c = strchr(source, '#')) == NULL) {
    sourcePath = source;
    sourceServer = NULL;
    sourceType = ELBASE_SCP;
  }
  else if((sourcePath = strchr(c + 1, '#')) == NULL) {
    *c = '\0';
    sourcePath = c + 1;
    sourceServer = source;
    sourceType = ELBASE_SCP;
  }
  else {
    *c = '\0';
    *(sourcePath++) = '\0';
    sourceServer = c + 1;
    for(sourceType = (ELBASE_StorageServiceTypes)0;
        SERVER_TYPE_NAMES[sourceType] != NULL &&
        strcmp(source, SERVER_TYPE_NAMES[sourceType]) != 0;
        sourceType++)
      ; /* Nothing to do. */
    if(SERVER_TYPE_NAMES[sourceType] == NULL) {
      fprintf(stderr, "Unknown server type %s\n", source);
      return 1;
    }
  }
  dest = strdup(ASENV_SwitchValueN(pa, "", 2, NULL));
  if((c = strchr(dest, '#')) == NULL) {
    destPath = dest;
    destServer = NULL;
    destType = ELBASE_SCP;
  }
  else if((destPath = strchr(c + 1, '#')) == NULL) {
    *c = '\0';
    destPath = c + 1;
    destServer = dest;
    destType = sourceType;
  }
  else {
    *c = '\0';
    *(destPath++) = '\0';
    destServer = c + 1;
    for(destType = (ELBASE_StorageServiceTypes)0;
        SERVER_TYPE_NAMES[destType] != NULL &&
        strcmp(dest, SERVER_TYPE_NAMES[destType]) != 0;
        destType++)
      ; /* Nothing to do. */
    if(SERVER_TYPE_NAMES[destType] == NULL) {
      fprintf(stderr, "Unknown server type %s\n", dest);
      return 1;
    }
  }
  scriptgen = strcmp(SwitchValue(pa, "--scriptgen", "no"), "yes") == 0;
  ParsedArgvFree(pa);

  if(scriptgen) {
    char *script = ELBASE_ScriptForCopy
      (sourceType, sourceServer, sourcePath, destType, destServer, destPath);
    if(script == NULL) {
      fprintf(stderr, "Script generation failed\n");
      return 1;
    }
    fprintf(stdout, "%s\n", script);
    free(script);
  }
  else if(!ELBASE_CopyFile(sourceType, sourceServer, sourcePath, destType,
                      destServer, destPath, &pid)) {
    fprintf(stderr, "Copy failed\n");
    return 1;
  }
  else if(!ELBASE_Poll(pid, 1, &status)) {
    fprintf(stderr, "Poll failed\n");
    return 1;
  }

  free(source);
  free(dest);
  return status;

}
