/* $Id: egad.c,v 1.7 2004/03/16 18:55:19 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This program provides a command-line interface to Elagi's gmingrp services.
  * Usage:
  *
  *   eg <port>
  *
  * The program is interactive.  It supports the following commands.
  *
  * add [computeService#]computeServer [[storageService#]storageServer]
  *   Add #computeServer# to the gmind group.  #computeService# defaults to
  *   ssh, #storageServer# to #computeServer#, and #storageService# to scp.
  *
  * check [attribute ...]
  *   Display the current values of attributes reported by group members.
  *
  * quit
  *   Shut down the gmind group and quit the program.
  */


#include <stdio.h>  /* fprintf */
#include <stdlib.h> /* strtol */
#include <string.h>
#include <ipseed.h>
#include "elbase.h"
#include "gmingrp.h"


static void
PrintAttr(const char *message,
          const char *attrName,
          const char *indent) {
  const char *c = strstr(message, attrName);
  if(c == NULL || (c = strchr(c, '=')) == NULL)
    return;
  fprintf(stdout, "%s", indent);
  for(c += 2; *c != '"' && *c != '\''; c++)
    fprintf(stdout, "%c", *c);
}


int
main(int argc,
     char **argv) {

  ASIP_Address addr;
  char *c;
  char *computeServer;
  ELBASE_ComputeServiceTypes computeType;
  char command[256];
  GMINGRP_GroupInfo group;
  char *message;
  char *metricName;
  ASIP_Port port;
  ASIP_Socket sock;
  char *storageServer;
  ELBASE_StorageServiceTypes storageType;

  if(argc != 2) {
    fprintf(stderr, "usage: egad <port>\n");
    return 1;
  }

  ASIP_AddressValue("localhost", &addr);
  port = (ASIP_Port)strtol(argv[1], NULL, 10);
  if((group = GMINGRP_GroupNew(port)) == NULL) {
    fprintf(stderr, "Unable to get group with port %d\n", (int)port);
    return 1;
  }

  while(1) {
    fprintf(stdout, "> ");
    fflush(stdout);
    if(fgets(command, sizeof(command), stdin) == NULL)
      break;
    if((c = strchr(command, '\n')) != NULL)
      *c = '\0';
    if(strcmp(command, "quit") == 0)
      break;
    if(strncmp(command, "add ", 4) == 0) {
      computeServer = command + 4;
      while(*computeServer == ' ')
        computeServer++;
      if((c = strchr(computeServer, ' ')) != NULL) {
        *c = '\0';
        while(*(++c) == ' ')
          ; /* empty */
      }
      storageServer = c == NULL || *c == '\0' ? computeServer : c;
      computeType = ELBASE_SSH;
      if((c = strchr(computeServer, '#')) != NULL) {
        *c = '\0';
        if(storageServer == computeServer)
          storageServer = c + 1;
        computeServer = c + 1;
      }
      storageType = ELBASE_SCP;
      if((c = strchr(storageServer, '#')) != NULL) {
        *c = '\0';
      }
      storageType = ELBASE_SCP;
      if(!GMINGRP_GroupAdd
           (group, storageType, storageServer, computeType, computeServer))
        fprintf(stdout, "failed\n");
      else
        fprintf(stdout, "ok\n");
    }
    else if(strcmp(command, "check") == 0 ||
            strncmp(command, "check ", 6) == 0) {
      if((sock = ASIP_ConnectToPort(addr, port)) == ASIP_NO_SOCKET) {
        fprintf(stderr, "Unable to connect to local daemon\n");
        break;
      }
      for(metricName = command + 5; *metricName == ' '; metricName++)
        ; /* empty */
      while(ASIP_ReceiveStringTerminated(sock, "\n", &message)) {
        if(strstr(message, "<HOST") != NULL)
          PrintAttr(message, "NAME", "\n");
        else if(strstr(message, "<METRIC") != NULL &&
                strstr(message, metricName) != NULL) {
          PrintAttr(message, "NAME", "\n  ");
          PrintAttr(message, "VAL", " ");
        }
        free(message);
      }
      fprintf(stdout, "\n");
      ASIP_Disconnect(&sock);
    }
  }

  GMINGRP_GroupFree(group);
  return 0;

}
