/* $Id: batchtest.c,v 1.5 2004/03/18 23:52:59 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI batch scheduler submission services
  * found in elbase.  It takes no arguments, but it does read server
  * information from the file servers.xml (see testutils.h) if available.
  */

#include <stdio.h>  /* File functions */
#include <oshseed.h>
#define ASPROP_SHORT_NAMES
#include <propertyseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "config.h"
#include "elbase.h"
#include "testutils.h"


int
main(int argc,
     char **argv) {

  char **attrs;
  XmlNode doc;
  FILE *f;
  unsigned i;
  unsigned long idx;
  XmlNode n;
  XmlNode opt;
  int passed;
  ELBASE_Process pid;
  PropertyList pl;
  const char *rmArgs[3] = {"-f", NULL, NULL};
  ELBASE_SchedulerServiceTypes scheduler;
  const char *server;
  XmlNodeList servers;
  ELBASE_ComputeServiceTypes service;
  int status;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  servers = DocumentGetElementsByTagName(doc, "host");
  rmArgs[1] = DEST_PATHS[0];

  for(i = 0; (n = NodeListItem(servers, i)) != NULL; i++) {

    /* Skip to the next host with a batch scheduler subtag. */
    n = NodeFirstChild(n);
    if(n == NULL || !StrArrayFind(COMPUTE_TAGS, NodeName(n), &idx))
      continue;
    server = ElementGetAttribute(n, "server");
    service = (ELBASE_ComputeServiceTypes)idx;
    n = NodeNextSibling(n);
    if(n == NULL || !StrArrayFind(SCHEDULER_TAGS, NodeName(n), &idx))
      continue;
    scheduler = (ELBASE_SchedulerServiceTypes)idx;

    /* Collect any scheduler options. */
    pl = PropertyListNew();
    for(opt = NodeFirstChild(n); opt != NULL; opt = NodeNextSibling(opt)) {
      const char *name;
      const char *value;
      if(strcmp(NodeName(opt), "option") != 0 ||
         (name = ElementGetAttribute(opt, "name")) == NULL ||
         (value = ElementGetAttribute(opt, "value")) == NULL)
        continue;
      SetProperty(&pl, name, value);
    }
    /* Add some defaults. */
    if(FindPropertyByName(pl, ELBASE_NODES) == NULL)
      SetProperty(&pl, ELBASE_NODES, "1");
    SetProperty(&pl, ELBASE_PROCESSES, "1");
    if(FindPropertyByName(pl, ELBASE_TIME) == NULL)
      SetProperty(&pl, ELBASE_TIME, "5");
    attrs = EnvironFromPropertyList(pl);
    PropertyListFree(&pl);

    /* Run test: submit ls w/redirected out, then delete output file. */
    passed = ELBASE_Submit(service, server, scheduler, (const char **)attrs,
                           "ls", NULL, NULL, NULL, DEST_PATHS[0], NULL, NULL,
                           &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0 &&
             ELBASE_Spawn(service, server, "rm", NULL, rmArgs, NULL, NULL,
                          NULL, NULL, &pid) &&
             ELBASE_Poll(pid, 1, &status) &&
             status == 0;
    fprintf(stdout, "%s scheduler via %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    StrArrayFree(attrs);

  }

  /* Report untested batch services. */
  for(i = 0; i < ELBASE_SCHEDULER_SERVICE_COUNT; i++) {
    if(FindNodeByChildTag(doc, SCHEDULER_TAGS[i]) == NULL)
      fprintf(stdout, "N/A schedule via %s\n", SCHEDULER_TAGS[i]);
  }

  return 0;

}
