/* $Id: gmintest.c,v 1.4 2004/03/15 16:55:11 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI minimum Ganglia services found in
  * gmingrp.  It takes no arguments, but it does read server information from
  * the file servers.xml (see testutil.h) if available.
  */

#include <stdio.h>  /* File functions */
#include <oshseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include <vectorseed.h>
#include "config.h"
#include "elbase.h"
#include "elmetric.h"
#include "gmingrp.h"
#include "testutils.h"


#define TEST_PORT 5656


int
main(int argc,
     char **argv) {

  char *diskName;
  XmlNode diskNode;
  const char *diskServer;
  ELBASE_StorageServiceTypes diskService;
  XmlNode doc;
  FILE *f;
  GMINGRP_GroupInfo group;
  int hostCount;
  XmlNode hostNode;
  const char *hostServer;
  ELBASE_ComputeServiceTypes hostService;
  unsigned i;
  unsigned long idx;
  ELMETRIC_Key *keys;
  XmlNodeList servers;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  if((group = GMINGRP_GroupNew(TEST_PORT)) == NULL) {
    fprintf(stderr, "Unable to create gmin group\n");
    NodeFree(doc);
    return 1;
  }

  servers = DocumentGetElementsByTagName(doc, "host");
  hostCount = 1;

  for(i = 0; (hostNode = NodeListItem(servers, i)) != NULL; i++) {

    if(*(diskName = ElementGetAttribute(hostNode, "disk")) == '\0')
      continue;
    hostNode = NodeFirstChild(hostNode);
    if(hostNode == NULL ||
       !StrArrayFind(COMPUTE_TAGS, NodeName(hostNode), &idx) ||
       NodeNextSibling(hostNode) != NULL) /* Batch host */
      continue;

    hostServer = ElementGetAttribute(hostNode, "server");
    hostService = (ELBASE_ComputeServiceTypes)idx;

    if((diskNode = FindElementByAttribute(doc, "id", diskName)) == NULL ||
       (diskNode = NodeFirstChild(diskNode)) == NULL ||
       !StrArrayFind(STORAGE_TAGS, NodeName(diskNode), &idx)) {
      fprintf(stdout, "Error: missing disk '%s'\n", diskName);
      continue;
    }

    diskServer = ElementGetAttribute(diskNode, "server");
    diskService = (ELBASE_StorageServiceTypes)idx;

    if(!GMINGRP_GroupAdd(group, diskService, diskServer, hostService,
                         hostServer)) {
      fprintf(stderr, "Unable to add %s to group\n", hostServer);
      continue;
    }

    hostCount++;

  }

  keys = ELMETRIC_GetAllKeys(ELMETRIC_GANGLIA, GMINGRP_GroupServer(group));
  if(keys == NULL)
    fprintf(stderr, "Unable to retrieve keys\n");
  else {
    for(i = 0; keys[i] != NULL; i++) {
      ELMETRIC_Key k = keys[i];
      if(ELMETRIC_GetKeyMetric(k) == ELMETRIC_CPU_LOAD) {
        ELMETRIC_Metric m;
        if((m = ELMETRIC_GetMetric(k)) == NULL)
          fprintf(stderr, "Unable to retrieve metric for %s\n",
                  ELMETRIC_GetKeyEntity(k));
        else
          ELMETRIC_FreeMetric(m);
        hostCount--;
      }
      ELMETRIC_FreeKey(k);
    }
    free(keys);
  }

  fprintf(stdout, "%s gmingrp info retrieval\n",
          hostCount == 0 ? "Pass" : "Fail");

  GMINGRP_GroupFree(group);
  NodeFree(doc);
  return 0;

}
