/* $Id: metrictest.c,v 1.6 2004/03/26 06:52:53 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

 /*
  * This is a test program for the ELAGI process batch scheduler services found
  * in elbase.  It takes no arguments, but it does read server information from
  * the file servers.xml if available.
  */

#include <stdio.h>  /* File functions */
#include <stdlib.h> /* free */
#include <oshseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "config.h"
#include "elbase.h"
#include "elmetric.h"
#include "testutils.h"


int
main(int argc,
     char **argv) {

  XmlNode doc;
  FILE *f;
  unsigned i;
  unsigned long idx;
  unsigned j;
  ELMETRIC_Key k;
  ELMETRIC_Key *keys;
  ELMETRIC_Metric m;
  XmlNode n;
  int passed;
  const char *server;
  XmlNodeList servers;
  ELMETRIC_MetricServiceTypes service;

  if((f = fopen("servers.xml", "r")) == NULL)
    doc = ImplementationCreateDocument(NULL, NULL, "doc", NULL);
  else {
    doc = ParseServers(f, stderr);
    fclose(f);
    if(doc == NULL)
      return 1;
  }
 
  servers = DocumentGetElementsByTagName(doc, "infosource");

  for(i = 0; (n = NodeListItem(servers, i)) != NULL; i++) {

    n = NodeFirstChild(n);
    if(n == NULL || !StrArrayFind(METRIC_TAGS, NodeName(n), &idx))
      continue;
    server = ElementGetAttribute(n, "server");
    service = (ELMETRIC_MetricServiceTypes)idx;
    keys = ELMETRIC_GetAllKeys(service, server);
    passed = keys != NULL && keys[0] != NULL;
    fprintf(stdout, "%s keys from %s (%s)\n", passed ? "Pass" : "Fail",
            NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
    if(keys != NULL) {
      k = keys[0];
      m = ELMETRIC_GetMetric(k);
      passed = m != NULL;
      fprintf(stdout, "%s metric from %s (%s)\n", passed ? "Pass" : "Fail",
              NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
      if(m != NULL)
        ELMETRIC_FreeMetric(m);
      k = ELMETRIC_GetKey
       (service, server, ELMETRIC_GetKeyEntity(k), ELMETRIC_GetKeyMetric(k));
      passed = k != NULL;
      fprintf(stdout, "%s key from %s (%s)\n", passed ? "Pass" : "Fail",
              NodeName(n), ElementGetAttribute(NodeParentNode(n), "id"));
      if(k != NULL)
        ELMETRIC_FreeKey(k);
      for(j = 0; keys[j] != NULL; j++)
        ELMETRIC_FreeKey(keys[j]);
      free(keys);
    }

  }

  for(i = 0; i < ELMETRIC_METRIC_SERVICE_COUNT; i++) {
    if(FindNodeByChildTag(doc, METRIC_TAGS[i]) == NULL)
      fprintf(stdout, "N/A metrics from %s\n", METRIC_TAGS[i]);
  }

  return 0;

}
