/* $Id: testutils.c,v 1.5 2004/03/24 19:41:51 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include <stdlib.h> /* atoi */
#include <string.h> /* strlen */
#define ASXML_SHORT_NAMES
#include <xmlseed.h>
#include "testutils.h"


const char *COMPUTE_TAGS[ELBASE_COMPUTE_SERVICE_COUNT + 1] = {
  "fork", "gram", "netsolve", "rash", "ssh", NULL
};
const char *METRIC_TAGS[ELMETRIC_METRIC_SERVICE_COUNT + 1] = {
  "local", "ganglia", "mds", "nws", NULL
};
const char *SCHEDULER_TAGS[ELBASE_SCHEDULER_SERVICE_COUNT + 1] = {
  "shell", "condor", "dqs", "loadleveler", "lsf", "pbs", "sge", NULL
};
const char *STORAGE_TAGS[ELBASE_STORAGE_SERVICE_COUNT + 1] = {
  "cp", "ftp", "gass", "scp", "sftp", "srb", NULL
};
const char *DEST_PATHS[4] =
  {"elagiTestDest0", "elagiTestDest1", "elagiTestDest2", NULL};
const char *SOURCE_PATHS[4] =
  {"elagiTestSrc0", "elagiTestSrc1", "elagiTestSrc2", NULL};


int
CheckTestFile(const char *path) {
  FILE *f;
  unsigned i;
  char line[256];
  if((f = fopen(path, "r")) == NULL)
    return 0;
  for(i = 0; fgets(line, sizeof(line), f) != NULL && atoi(line) == (int)i; i++)
    ; /* empty */
  fclose(f);
  return i == FILE_SIZE;
}


int
CreateTestFile(const char *path) {
  FILE *f;
  unsigned i;
  if((f = fopen(path, "w")) == NULL)
    return 0;
  for(i = 0; i < FILE_SIZE; i++)
    fprintf(f, "%d\n", i);
  fclose(f);
  return 1;
}


XmlElement
FindElementByAttribute(XmlNode root,
                       const char *attrName,
                       const char *attrValue) {
  XmlNode n;
  if(NodeType(root) == ELEMENT_NODE &&
     strcmp(ElementGetAttribute(root, (char *)attrName), attrValue) == 0)
    return root;
  for(n = NodeFirstChild(root); n != NULL; n = NodeNextSibling(n))
    if((root = FindElementByAttribute(n, attrName, attrValue)) != NULL)
      return root;
  return NULL;
}


XmlNode
FindNodeByChildTag(XmlNode root,
                   const char *tag) {
  XmlNode n;
  for(n = NodeFirstChild(root); n != NULL; n = NodeNextSibling(n))
    if(strcmp(NodeName(n), tag) == 0)
      return root;
  for(n = NodeFirstChild(root); n != NULL; n = NodeNextSibling(n))
    if((root = FindNodeByChildTag(n, tag)) != NULL)
      return root;
  return NULL;
}


XmlNode
ParseServers(FILE *f,
             FILE *err) {

  XmlParseErrorCodes error;
  unsigned i;
  unsigned long offset;
  char line[256];
  XmlNode result;

  result = XmlFromStream
    ((XmlGet)fgetc, f, NULL, 1, 1, NULL, &error, &offset);
  if(result != NULL || err == NULL)
    return result;
  fseek(f, 0, SEEK_SET);
  for(i = 1; fgets(line, sizeof(line), f) != NULL && offset > strlen(line); i++)
    offset -= strlen(line);
  fprintf(err, "Syntax error on line %d\n%s\n%*s\n", i, line, (int)offset, "^");
  return NULL;

}
