/* $Id: testutils.h,v 1.2 2004/03/16 18:56:23 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */

/*
 * This package contains utilities and declarations useful to multiple elagi
 * test programs.  Parsing of the test programs' input file, servers.xml, is
 * done here.  The format of this file is
 *   <elagi>
 *     <storage>
 *       <disk id='string'>
 *         <scp server='string'/> <!-- or cp, ftp, gass, sftp, srb -->
 *       </disk>
 *     </storage>
 *     <compute>
 *       <host id='string' disk='string'>
 *         <ssh server='string'/> <!-- or fork, globus -->
 *         <condor> <!-- or dqs, loadleveler, lsf, pbs, sge -->
 *           <option name='string' value='string'/>
 *         </condor>
 *       </host>
 *     </compute>
 *     <gridinfo>
 *       <infosource>
 *         <ganglia server='string'/> <!-- or mds, nws -->
 *       </infosource>
 *     </gridinfo>
 *   </elagi>
 * Each program only uses a subset of the information from servers.xml.
 */

#include <stdio.h>
#include <xmlseed.h>
#include "elbase.h"
#include "elmetric.h"


/* Searchable arrays of XML tag names corresponding to elagi enums. */
extern const char *COMPUTE_TAGS[ELBASE_COMPUTE_SERVICE_COUNT + 1];
extern const char *METRIC_TAGS[ELMETRIC_METRIC_SERVICE_COUNT + 1];
extern const char *SCHEDULER_TAGS[ELBASE_SCHEDULER_SERVICE_COUNT + 1];
extern const char *STORAGE_TAGS[ELBASE_STORAGE_SERVICE_COUNT + 1];

/* Paths for file copying. */
extern const char *DEST_PATHS[4];
extern const char *SOURCE_PATHS[4];
#define FILE_SIZE 100


/* Returns 1 if file #path# contains correct content, else 0. */
int
CheckTestFile(const char *path);


/*
 * Writes data to the file at #path# for later checking by CheckTestFile.
 * Returns 1 if successful, else 0.
 */
int
CreateTestFile(const char *path);


/*
 * Returns the first subnode of #root# that has an attribute with name
 * #attrName# and value #attrValue#, NULL if none.
 */
ASXML_XmlElement
FindElementByAttribute(ASXML_XmlNode root,
                       const char *attrName,
                       const char *attrValue);

/*
 * Returns the first subnode of #root# that has a child named #tag#, NULL if
 * none.
 */
ASXML_XmlNode
FindNodeByChildTag(ASXML_XmlNode root,
                   const char *tag);

/*
 * Parses and returns the XML contained in #f#.  On error, returns NULL and
 * prints a message to #err# if it is not NULL.
 */
ASXML_XmlNode
ParseServers(FILE *f,
             FILE *err);
