/* $Id: bulkcopy.c,v 1.8 2004/03/18 23:52:59 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <oshseed.h>
#include <stdio.h>    /* file functions */
#include <stdlib.h>   /* free malloc */
#include <string.h>   /* strdup */
#include <fileseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASV_SHORT_NAMES
#include <vectorseed.h>
#include "bulkcopy.h"


struct ELBC__BulkLink {
  ELBASE_StorageServiceTypes sourceStorageService;
  char *sourceStorageServer;
  ELBASE_ComputeServiceTypes sourceComputeService;
  char *sourceComputeServer;
  ELBASE_StorageServiceTypes destStorageService;
  char *destStorageServer;
  ELBASE_ComputeServiceTypes destComputeService;
  char *destComputeServer;
};


void
ELBC_BulkLinkFree(ELBC_BulkLink link) {
  free(link->sourceStorageServer);
  free(link->sourceComputeServer);
  free(link->destStorageServer);
  free(link->destComputeServer);
  free(link);
}


ELBC_BulkLink
ELBC_BulkLinkNew(ELBASE_StorageServiceTypes sourceStorageService,
                 const char *sourceStorageServer,
                 ELBASE_ComputeServiceTypes sourceComputeService,
                 const char *sourceComputeServer,
                 ELBASE_StorageServiceTypes destStorageService,
                 const char *destStorageServer,
                 ELBASE_ComputeServiceTypes destComputeService,
                 const char *destComputeServer) {
  ELBC_BulkLink result;
  if(destComputeService != ELBASE_FORK && destComputeService != ELBASE_SSH)
    return NULL;
  result = (ELBC_BulkLink)malloc(sizeof(struct ELBC__BulkLink));
  result->sourceStorageService = sourceStorageService;
  result->sourceStorageServer = strdup(sourceStorageServer);
  result->sourceComputeService = sourceComputeService;
  result->sourceComputeServer = strdup(sourceComputeServer);
  result->destStorageService = destStorageService;
  result->destStorageServer = strdup(destStorageServer);
  result->destComputeService = destComputeService;
  result->destComputeServer = strdup(destComputeServer);
  return result;
}


int
ELBC_BulkCopy(ELBC_BulkLink link,
              const char **sourcePaths,
              const char **destPaths,
              unsigned pathCount,
              ELBASE_Process *pid) {

  const char **args;
  FILE *f;
  unsigned i;
  int result;
  int savedFd;
  char *script;
  char *scriptPath = ASFILE_UniqueFilePath("elbc_bulkscript");
  int status;
  int tarPipe[2];
  char *untarPath;

  if((f = fopen(scriptPath, "w")) == NULL) {
    free(scriptPath);
    return 0;
  }

  untarPath = ASFILE_UniqueFilePath("elbc_bulk");
  script = StrAppend(NULL,
    "rm -f $0\n",
    "mkdir ", untarPath, " || exit 1\n",
    "cd ", untarPath, " || exit 2\n",
    "tar x || exit 3\n",
    "cd .. || exit 4\n",
    NULL);
  for(i = 0; i < pathCount; i++)
    script = StrAppend
      (script, "mv -f ", untarPath, "/",
       sourcePaths[i] + (*sourcePaths[i] == '/' ? 1 : 0), " ", destPaths[i],
       " || exit 5\n", NULL);
  script = StrAppend(script,
    "rm -fr ", untarPath, "\n",
    NULL);
  free(untarPath);
  fprintf(f, "%s", script);
  fclose(f);
  free(script);

  if(link->destStorageService != ELBASE_CP) {
    result =
      ELBASE_CopyFile(ELBASE_CP, NULL, scriptPath, link->destStorageService,
                      link->destStorageServer, scriptPath, pid) &&
      ELBASE_Poll(*pid, 1, &status) &&
      status == 0;
    unlink(scriptPath);
    if(!result) {
      free(scriptPath);
      return 0;
    }
  }

  pipe(tarPipe);
  savedFd = dup(1);
  dup2(tarPipe[1], 1);
  args = VectorNew(const char *);
  Append(args, "cf");
  Append(args, "-");
  for(i = 0; i < pathCount; i++)
    Append(args, sourcePaths[i]);
  Append(args, NULL);
  result = ELBASE_Spawn(link->sourceComputeService, link->sourceComputeServer,
                        "tar", NULL, args, NULL, NULL, NULL, NULL, pid);
  VectorFree(args);
  dup2(savedFd, 1);
  close(savedFd);
  close(tarPipe[1]);

  if(result) {
    savedFd = dup(0);
    dup2(tarPipe[0], 0);
    args = VectorNew(const char *);
    Append(args, scriptPath);
    Append(args, NULL);
    result = ELBASE_Spawn(link->destComputeService, link->destComputeServer,
                          "/bin/sh", NULL, args, NULL, NULL, NULL, NULL, pid);
    VectorFree(args);
    dup2(savedFd, 0);
    close(savedFd);
  }

  close(tarPipe[0]);
  free(scriptPath);
  return result;

}
