/* $Id: rashgrp.c,v 1.49 2004/03/24 19:40:19 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <oshseed.h>
#include <stdio.h> /* file functions */
#include <stdlib.h> /* atoi calloc free strtol system */
#include <string.h> /* strlen */
#include <fileseed.h>
#include <ipseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#include "rashscripts.h"
#include "rashgrp.h"


#define CWD "." ASOSH_FILE_PATH_SEPARATOR


struct RASHGRP__GroupInfo {
  char *agentPath;           /* remote path to rasha */
  ELBASE_Process agentPid;   /* pid of rasha submission process */
  char *proxyContact;        /* contact string for rashd proxy */
  char *proxyPath;           /* remote path to rashd */
  ELBASE_Process proxyPid;   /* pid of process running rashd */
  ELBASE_Process readyPid;   /* pid of process waiting for submission to run */
  ELBASE_ComputeServiceTypes type; /* remote compute type */
  char *server;              /* remote compute contact string */
  char *wd;                  /* remote wd */
};


static char *localRashPath = NULL;
static char *localRashaPath = NULL;
static char *localRashdPath = NULL;


/*
 * Copies the file specified in #localPath# to #remotePath# via the #type#
 * server #server#.  Blocks until the copy is complete.  Returns 1 if
 * successful, else 0.
 */
static int
DoFileCopy(const char *localPath,
           ELBASE_StorageServiceTypes type,
           const char *server,
           const char *remotePath) {
  ELBASE_Process pid;
  int status;
  return ELBASE_CopyFile
    (ELBASE_CP, "", localPath, type, server, remotePath, &pid) &&
    ELBASE_Poll(pid, 1, &status) &&
    status == 0;
}


/* Calls ELBASE_Poll on *#pid# and sets it to NULL if the process is gone. */
static int
PollProcess(ELBASE_Process *pid,
            int wait,
            int *status) {
  if(!ELBASE_Poll(*pid, wait, status))
    return 0;
  *pid = NULL;
  return 1;
}


int
RASHGRP_AgentsPending(RASHGRP_GroupInfo group,
                      int *status) {
  if(status != NULL)
    *status = 0;
  return group->agentPid != NULL && !PollProcess(&group->agentPid, 0, status) &&
         group->readyPid != NULL && !PollProcess(&group->readyPid, 0, status);
}


int
RASHGRP_AgentsRunning(RASHGRP_GroupInfo group) {
  return group->agentPid != NULL && !PollProcess(&group->agentPid, 0, NULL) &&
         (group->readyPid == NULL || PollProcess(&group->readyPid, 0, NULL));
}


char **
RASHGRP_GetAgentContacts(RASHGRP_GroupInfo group) {

  char buffer[512];
  char *command;
  FILE *f;
  unsigned i;
  char *output = NULL;
  char **result = NULL;

  /* Gather the output from a rash "who" command directed at the proxy. */
  command = StrAppend
    (NULL, "perl ", localRashPath, " 0@", group->proxyContact, " who 2>&1",
     NULL);
  f = popen(command, "r");
  free(command);
  if(f == NULL)
    return NULL;
  while(fgets(buffer, sizeof(buffer), f) != NULL)
    output = StrAppend(output, buffer, NULL);
  if(pclose(f) == 0)
    result = StrSplit(output, NULL);
  if(output != NULL)
    free(output);
  if(result == NULL)
    return NULL;
  /* Throw away the proxy and client names. */
  free(result[0]);
  for(i = 0; result[i] != NULL; i++)
    result[i] = result[i + 1];
  i -= 2;
  free(result[i]);
  result[i] = NULL;
  /* Append proxy contact to agent contact names. */
  for(i = 0; result[i] != NULL; i++)
    result[i] = StrAppend(result[i], "@", group->proxyContact, NULL);
  return result;

}


void
RASHGRP_GroupFree(RASHGRP_GroupInfo group) {
  const char *args[4] = {"-f", NULL, NULL, NULL};
  ELBASE_Process pid;
  RASHGRP_StopAgents(group);
  RASHGRP_StopProxy(group);
  args[1] = group->agentPath;
  args[2] = group->proxyPath;
  if(ELBASE_Spawn(group->type, group->server, "rm", NULL, args,
                  NULL, NULL, NULL, NULL, &pid))
    ELBASE_Poll(pid, 1, NULL);
  free(group->agentPath);
  free(group->proxyContact);
  free(group->proxyPath);
  free(group->server);
  free(group->wd);
  free(group);
}


RASHGRP_GroupInfo
RASHGRP_GroupNew(ELBASE_StorageServiceTypes storageType,
                 const char *storageServer,
                 ELBASE_ComputeServiceTypes computeType,
                 const char *computeServer,
                 unsigned proxyPort,
                 const char *workingDirectory) {

  char *c;
  char image[64];
  char *remoteRashaPath;
  char *remoteRashdPath;
  RASHGRP_GroupInfo result;
  char *uniquePath;

  if(computeType == ELBASE_NETSOLVE || computeType == ELBASE_RASH)
    return NULL; /* Not supported. */

  c = StrAppend(NULL, workingDirectory, "/rash", NULL);
  uniquePath = ASFILE_UniqueFilePath(c);
  free(c);
  remoteRashaPath = StrAppend(NULL, uniquePath, "a", NULL);
  remoteRashdPath = StrAppend(NULL, uniquePath, "d", NULL);
  free(uniquePath);
  if(!DoFileCopy(localRashaPath, storageType, storageServer, remoteRashaPath) ||
     !DoFileCopy(localRashdPath, storageType, storageServer, remoteRashdPath)) {
    free(remoteRashaPath);
    free(remoteRashdPath);
    return NULL;
  }

  result = calloc(1, sizeof(struct RASHGRP__GroupInfo));
  result->agentPath = remoteRashaPath;
  result->agentPid = NULL;
  if(computeType == ELBASE_FORK)
    result->proxyContact = ASIP_MyMachineName();
  else if(computeType == ELBASE_SSH)
    result->proxyContact = strdup
     ((c = strchr((char *)computeServer, '@')) != NULL ? c + 1 : computeServer);
  else if(computeType == ELBASE_GRAM) {
    result->proxyContact = strdup(computeServer);
    if((c = strchr(result->proxyContact, '/')) != NULL)
      *c = '\0';
  }
  if((c = strchr(result->proxyContact, ':')) != NULL)
    *c = '\0';

  sprintf(image, "%d", (int)proxyPort);
  result->proxyContact = StrAppend(result->proxyContact, ":", image, NULL);
  result->proxyPath = remoteRashdPath;
  result->proxyPid = NULL;
  result->readyPid = NULL;
  result->type = computeType;
  result->server = strdup(computeServer);
  result->wd = strdup(workingDirectory);
  return result;

}


int
RASHGRP_Init(void) {

  FILE *f;
  unsigned i;
  char image[16];
  char *paths[3];
  char *script;

  sprintf(image, "%d", (int)getpid());
  paths[0] = localRashPath = StrAppend(NULL, CWD "rash", image, NULL);
  paths[1] = localRashaPath = StrAppend(NULL, CWD "rasha", image, NULL);
  paths[2] = localRashdPath = StrAppend(NULL, CWD "rashd", image, NULL);

  for(i = 0; i < 3; i++) {
    if((f = fopen(paths[i], "w")) == NULL)
      return 0;
    script = StrArrayJoin
      (i == 0 ? rash_script : i == 1 ? rasha_script : rashd_script, "\n");
    fprintf(f, "%s\n", script);
    fclose(f);
    free(script);
  }
  script = StrAppend(NULL, "perl ", localRashPath, NULL);
  ELBASE_SetComputeCommand(ELBASE_RASH, script);
  free(script);
  return 1;

}


int
RASHGRP_ProxyRunning(RASHGRP_GroupInfo group) {
  return group->proxyPid != NULL && !PollProcess(group->proxyPid, 0, NULL);
}


int
RASHGRP_Shutdown(void) {
  unlink(localRashPath);
  unlink(localRashaPath);
  unlink(localRashdPath);
  return 1;
}


int
RASHGRP_StartProxy(RASHGRP_GroupInfo group) {

  ASIP_Address addr;
  const char *args[5] = {NULL, "-d", "-p", NULL, NULL};
  char *c = strchr(group->proxyContact, ':');
  ASIP_Port port;
  ASIP_Socket sock;

  args[0] = group->proxyPath;
  args[3] = c + 1;
  if(!ELBASE_Spawn(group->type, group->server, "perl", group->wd, args, NULL,
                   "/dev/null", "/dev/null", NULL, &group->proxyPid))
    return 0;
  *c = '\0';
  ASIP_AddressValue(group->proxyContact, &addr);
  port = (ASIP_Port)atoi(c + 1);
  *c = ':';
  while((sock = ASIP_ConnectToTcpPort(addr, port)) == ASIP_NO_SOCKET) {
    if(PollProcess(&group->proxyPid, 0, NULL))
      return 0;
    sleep(1);
  }
  ASIP_Disconnect(&sock);
  return 1;

}


void
RASHGRP_StopAgents(RASHGRP_GroupInfo group) {
  char **agents;
  char *command;
  unsigned i;
  
  /* Make sure the proxy forgets about any running agents. */
  if((agents = RASHGRP_GetAgentContacts(group)) != NULL) {
    for(i = 0; agents[i] != NULL; i++) {
      *strchr(agents[i], '@') = '\0';
      command = StrAppend
        (NULL, "perl ", localRashPath, " 0@", group->proxyContact, " stop ",
         agents[i], " > /dev/null 2>&1", NULL);
      system(command);
      free(command);
    }
    StrArrayFree(agents);
  }
  if(group->agentPid != NULL) {
    ELBASE_Kill(group->agentPid);
    PollProcess(&group->agentPid, 1, NULL);
  }
  if(group->readyPid != NULL) {
    ELBASE_Kill(group->readyPid);
    PollProcess(&group->readyPid, 1, NULL);
  }
}


void
RASHGRP_StopProxy(RASHGRP_GroupInfo group) {
  char *command;
  RASHGRP_StopAgents(group);
  command = StrAppend(NULL, "perl ", localRashPath, " 0@", group->proxyContact,
                      " stop _rashd", " 2>&1 > /dev/null", NULL);
  system(command);
  free(command);
  if(group->proxyPid != NULL) {
    /* Since the proxy killed itself, this Kill is harmlessly redundant. */
    ELBASE_Kill(group->proxyPid);
    PollProcess(&group->proxyPid, 1, NULL);
  }
}


int
RASHGRP_SubmitAgents(RASHGRP_GroupInfo group,
                     ELBASE_SchedulerServiceTypes scheduler,
                     const char **attributes) {
  const char *args[5] = {NULL};
  unsigned long i;
  char lastAgentId[20];
  /* Launch a process to make and monitor the submission.... */
  args[0] = group->agentPath;
  args[1] = group->proxyContact;
  if(attributes == NULL ||
     group->proxyPid == NULL ||
     !ELBASE_Submit(group->type, group->server, scheduler, attributes, "perl",
                    group->wd, args, NULL, NULL, NULL, NULL, &group->agentPid))
    return 0;
  /* ... and another to wait for all submitted agents to check in. */
  if(StrArrayNFind(attributes, StrArrayLen(attributes), ELBASE_NODES,
                   strlen(ELBASE_NODES), &i))
    sprintf(lastAgentId, "%ld",
            strtol(attributes[i] + strlen(ELBASE_NODES) + 1, NULL, 10) + 1);
  else
    sprintf(lastAgentId, "2");
  args[0] = localRashPath;
  args[1] = "-w";
  args[2] = StrAppend(NULL, lastAgentId, "@", group->proxyContact, NULL);
  args[3] = "quit";
  ELBASE_Fork("perl", NULL, args, NULL, NULL, NULL, NULL, &group->readyPid);
  free((char *)args[2]);
  return 1;
}
