/* $Id: taskpool.c,v 1.16 2004/01/12 19:12:23 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California.
 * All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies.
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office
 * 9500 Gilman Drive
 * 411 University Center
 * University of California
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fileseed.h>

#include "taskpool.h"

int
TP_WriteBatchFile(char **commands,
                  int nCommands,
                  const char *jobWorkingDir,
                  const char *path,
                  const char *exitCodeFile)
{
  FILE *f;
  int i;
  char *numfilename;
  char *c;

  if (nCommands <= 0)
    return 0;
  if ((!commands) || (!path))
    return 0;


  numfilename = malloc(strlen(path) + 24);
  c = strrchr(path, '/');
  if (c)
    sprintf(numfilename, "%s_next_task_index", c+1);
  else
    sprintf(numfilename, "next_task_index");

  f = fopen(path, "wb");
  if (!f)
    return 0;

 
  fprintf(f, 
    "#!/bin/sh\n"
    "\n"
    "###################################\n"
    "# Initialize variables\n"
    "numberFileName=\"%s\"\n"
    "outputFileName=\"%s\"\n"
    "index=0\n"
    "finished_a_task=0\n"
    "exit_code=-1\n"
    "\n"
    "cd %s\n\n"
    "if [ ! -f $numberFileName ]; then\n"
    "  echo 0 > $numberFileName\n"
    "fi\n"
    "\n", numfilename, exitCodeFile, jobWorkingDir);

  fprintf(f,
    "while [ $index -lt %d ]; do\n",
    nCommands);

  fprintf(f,
    "\n"
    "  ####################################################\n"
    "  # Acquire the lock (Create an exclusive lock file)\n"
    "  locked=0\n"
    "  while [ $locked -eq 0 ]; do\n"
    "    # Set the default file creation attributes to unreadable, unwritable\n"
    "    # then try to create a lock file.\n"
    "    perl -e \"umask(0777); exit open F, '> ' . '.lock.$numberFileName';\"\n"
    "    if [ $? -eq 1 ]; then\n"
    "      locked=1;\n"
    "    else\n"
    "      sleep 1\n"
    "    fi\n"
    "  done;\n"
    "\n"
    "  if [ $finished_a_task -eq 1 ]; then\n"
    "    echo \"$index\t$exit_code\" >> $outputFileName\n"
    "  fi\n"
    "\n");
  fprintf(f,
    "  # Pick an index out of the (lock-protected) number file, and\n"
    "  # increment the index for the next process.\n"
    "  if [ -f $numberFileName ]; then\n"
    "    index=`cat $numberFileName`\n"
    "  else\n"
    "    index=%d\n"
    "  fi\n"
    "  echo $(($index+1)) > $numberFileName\n"
    "\n"
    "  ##############################################\n"
    "  # Release the lock (delete the lock file)\n"
    "  rm -f \".lock.$numberFileName\"\n"
    "\n"
    "  ##############################################\n"
    "  # Do the work for a single index number.\n"
    "  case $index in\n", nCommands);

  for (i = 0; i < nCommands; i++)
  {
    fprintf(f,
        "      %d) %s;;\n",
        i,
        commands[i]);
  }

  fprintf(f, 
    "      *) rm $numberFileName;;\n"
    "  esac\n\n" 
    "  finished_a_task=1\n"
    "  exit_code=$?\n"
    "done\n"
    "exit 0");

  fclose(f);
  ASFILE_SetFilePermission(path, AS_OWNER, 1, 1, 1);
  ASFILE_SetFilePermission(path, AS_GROUP, 1, 1, 1);
  ASFILE_SetFilePermission(path, AS_OTHERS, 1, 0, 1);

  free(numfilename);
  return 1;  
}
