/* $Id: elmetric.h,v 1.7 2004/03/05 19:39:44 jhayes Exp $ */
/*
 * Copyright  2004 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef ELMETRIC_H
#define ELMETRIC_H

/*
 * This package provides facilities for retrieving metrics (measurements and
 * static data) about Grid resources using a variety of Grid services.
 */

#ifdef __cplusplus
extern "C" {
#endif

/* for GHS -START*/
/* Supported Grid metric services. */
typedef enum {
  ELMETRIC_LOCAL, ELMETRIC_GANGLIA, ELMETRIC_MDS, ELMETRIC_NWS,
  ELMETRIC_GHS,
  ELMETRIC_METRIC_SERVICE_COUNT
} ELMETRIC_MetricServiceTypes;

/* Supported Grid metrics. */
typedef enum {
  ELMETRIC_CPU_COUNT, ELMETRIC_CPU_LOAD, ELMETRIC_TCP_BANDWIDTH,
  ELMETRIC_TCP_LATENCY,
  ELMETRIC_JOB_ARRIVAL_RATE, ELMETRIC_SERVICE_TIME_DEVIATION, ELMETRIC_COMPUTING_CAPACITY,
  ELMETRIC_METRIC_COUNT
} ELMETRIC_MetricTypes;
/* for GHS -END*/

/* Key allowing retrieval of a particular metric from a particular service. */
typedef struct _ELMETRIC_KeyStruct *ELMETRIC_Key;
/* Metric retrieved by a particular key. */
typedef struct _ELMETRIC_MetricStruct *ELMETRIC_Metric;

/* Invalid key and metric values. */
#define ELMETRIC_NO_KEY 0
#define ELMETRIC_NO_METRIC 0


/* Frees #k# and any allocated memory it contains. */
void
ELMETRIC_FreeKey(ELMETRIC_Key k);


/* Frees #m# and any allocated memory it contains. */
void
ELMETRIC_FreeMetric(ELMETRIC_Metric m);


/*
 * Returns in a NULL-terminated, allocated array keys for all the metrics that
 * are available from the #service# server #server#.  Returns NULL on error.
 */
ELMETRIC_Key *
ELMETRIC_GetAllKeys(ELMETRIC_MetricServiceTypes service,
                    const char *server);


/*
 * Returns a key that can be passed to ELMETRIC_GetMetric in order to retrieve
 * the #metric# value for #entity# from the #service# server #server#.  Returns
 * ELMETRIC_NO_KEY if #server# does not provide this metric.
 */
ELMETRIC_Key
ELMETRIC_GetKey(ELMETRIC_MetricServiceTypes service,
                const char *server,
                const char *entity,
                ELMETRIC_MetricTypes metric);


/* Returns the entity passed when #k# was created. */
char *
ELMETRIC_GetKeyEntity(ELMETRIC_Key k);


/* Returns the metric passed when #k# was created. */
ELMETRIC_MetricTypes
ELMETRIC_GetKeyMetric(ELMETRIC_Key k);


/* Returns the server passed when #k# was created. */
char *
ELMETRIC_GetKeyServer(ELMETRIC_Key k);


/* Returns the service passed when #k# was created. */
ELMETRIC_MetricServiceTypes
ELMETRIC_GetKeyService(ELMETRIC_Key k);


/*
 * Returns the current metric associated with #k#, or ELMETRIC_NO_METRIC if the
 * metric cannot be determined.
 */
ELMETRIC_Metric
ELMETRIC_GetMetric(ELMETRIC_Key k);


/*
 * Returns the time (seconds since 1/1/1970) at which the value stored in #m#
 * should no longer be considered valid.
 */
unsigned long
ELMETRIC_GetMetricExpiration(ELMETRIC_Metric m);


/* Returns the key associated with #m#. */
ELMETRIC_Key
ELMETRIC_GetMetricKey(ELMETRIC_Metric m);


/* Returns the value associated with #m#. */
char *
ELMETRIC_GetMetricValue(ELMETRIC_Metric m);


/*
 * Returns 1 or 0 depending on whether or not #server# is a valid contact
 * specification for a #service# server.
 */
int
ELMETRIC_ValidServer(ELMETRIC_MetricServiceTypes service,
                     const char *server);


#ifdef __cplusplus
}
#endif


#endif
