/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<time.h>
#include "ghs.h"

double maxJobWorkload = 2000;

extern double GetParaCDF(double tasktime);

extern double GetParaMean(int numOfinte);
extern double GetParaVari(double paraMean, int numOfInte);

double utilPara[MAX_WS_NUM], servPara[MAX_WS_NUM], arriPara[MAX_WS_NUM], servstdPara[MAX_WS_NUM], workPara[MAX_WS_NUM], speedPara[MAX_WS_NUM];

char machname[MAX_WS_NUM][30];
char taskname[MAX_WS_NUM][30];

int maptab[MAX_WS_NUM][MAX_GJOB_NUM];

double currworkload[MAX_GJOB_NUM];

int wsNum;

double meantime, variance;


/* calcuate the mean+variance of the parallel task runtime
 * using model formulas 
 */
double 
EstiRuntime()
{
  int i = 0, j = 0;

  ReadTaskPara(utilPara, arriPara, servstdPara, workPara, wsNum);
  //read_taskpara(util, arri, servstd, work, j);
  meantime = GetParaMean(XNUM);
  //printf("the meantime is %f \n", meantime);
	
  variance = GetParaVari(meantime, XNUM);
  //printf("the variance is %f \n", variance);
  meantime = GetParaMean(XNUM);

}

/*  read resource parameters
 *	resource name, utilization, arrival rate, servicestd
 */
void 
GetMapTab(char *configFile) 
{
  int i = 0;
  FILE *stream;
  char line[100];
  float tmpUtil, tmpArri, tmpServ, tmpSpeed, tmpWorkload;

  if ( (stream = fopen(configFile,"r")) == NULL) 
  {
    printf("can't open the maptab file\n");
    exit(1);
  }

  while ( fgets(line, 100, stream) != NULL) 
  {
    sscanf(line, "%s %f %f %f %f %f", &machname[i], &tmpUtil, &tmpArri, &tmpServ, &tmpSpeed, &tmpWorkload);
    utilPara[i] = tmpUtil;
    arriPara[i] = tmpArri;
    servstdPara[i] = tmpServ;
    speedPara[i] = tmpSpeed;
    workPara[i] = tmpWorkload/speedPara[i];
    //printf(" %f %f %f %f \n", utilPara[i], arriPara[i], servstdPara[i], speedPara[i]);
    i++;
  }

  wsNum = i;
  fclose(stream);

}

/* write the task allocation result into a result file
 * which name is provided by argv[3]
 */

void 
OutputPredict(char *outputFile)
{
  int i = 0;
  FILE *stream;
  char line[100];
  float maxWork= 0, taskTime, timeInterval, newCDF;

  if ( (stream = fopen(outputFile,"w")) == NULL) 
  {
    printf("can't open output file\n");
    exit(1);
  }

  sprintf(line, "%f %f \n", meantime, sqrt(variance));
  fputs(line, stream);

  CalculateAB();

  for (i = 0; i < wsNum; i++)
  {
    if (maxWork < workPara[i]) maxWork = workPara[i];
  }

  timeInterval = (10 * maxWork)/PHASE_NUM;

  for (i = 0; i < PHASE_NUM; i++) 
  {
    taskTime = maxWork + timeInterval * i;
    //printf("the i is %d and the taskTime is %f %f \n", i, taskTime, timeInterval);
    newCDF = GetParaCDF(taskTime + timeInterval);

    if ( newCDF > 0.995) break;

    sprintf(line, "%f %f \n", taskTime + timeInterval, newCDF);
    fputs(line, stream);
  }

  fclose(stream);

}

int 
main(int argc, char **argv)
{
  int i;

  if (argc != 3)
  {
    printf("apppred mapinfo.txt result.txt \n");
    exit(1);
  }

  GetMapTab(argv[1]);
  EstiRuntime();
  OutputPredict(argv[2]);
  return(0);

}
