/* $Id: corestructures.c,v 1.56 2003/11/11 23:33:33 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <stdio.h>   /* sprintf */
#include <string.h>  /* memset */
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#define ASV_SHORT_NAMES
#include <vectorseed.h>
#include "misc.h"
#include "corestructures.h"


/*** DISK Functions ***/


void
Disk_Free(Disk d) {
  if(d->id != NULL)
    FREE(d->id);
  if(d->datadir != NULL)
    FREE(d->datadir);
  if(d->server != NULL)
    FREE(d->server);
  if(d->transfersCompleted != NULL)
    VectorFree(d->transfersCompleted);
  if(d->transfersInProgress != NULL)
    VectorFree(d->transfersInProgress);
  FREE(d);
}


Disk
Disk_New(char *id,
         char *datadir,
         ELBASE_StorageServiceTypes method,
         char *server) {
  Disk disk;
  disk = (Disk)MALLOC(sizeof(struct _Disk));
  disk->id = id;
  disk->datadir = datadir;
  disk->method = method;
  disk->server = server;
  disk->transfersCompleted = VectorNew(FileTransfer);
  disk->transfersInProgress = VectorNew(FileTransfer);
  return disk;
}


void
DiskList_Add(DiskList dl,
             Disk d) {
  ASH_PutMapping(dl, d->id, d);
}


void
DiskList_Free(DiskList dl,
              int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(dl, it)
      Disk_Free((Disk)ASH_IteratorValue(it));
  ASH_HashTableFree(dl);
}


/*** FILE Functions ***/


void
File_Free(File f) {
  if(f->name != NULL)
    FREE(f->name);
  if(f->copies != NULL)
    /* File copies have no meaning after the File has been destroyed */
    FileCopyList_Free(f->copies, 1);
  FREE(f);
}


File
File_New(char *name) {
  File file = (File)MALLOC(sizeof(struct _File));
  file->name = name;
  file->size = 0;
  file->outputOf = NULL;
  file->download = 0;
  file->transferable = 0;
  file->copies = FileCopyList_New(LINKED_LIST);
  return file;
}


void
FileList_Add(FileList fl,
             File f) {
  ASH_PutMapping(fl, f->name, f);
}

void
FileList_Free(FileList fl,
              int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(fl, it)
      File_Free((File)ASH_IteratorValue(it));
  ASH_HashTableFree(fl);
}


/*** FILE COPY Functions ***/


void
FileCopy_Free(FileCopy fc) {
  if(fc->path != NULL)
    FREE(fc->path);
  FREE(fc);
}


FileCopy
FileCopy_New(Disk disk,
             char *path) {
  FileCopy fc = (FileCopy)MALLOC(sizeof(struct _FileCopy));
  fc->disk = disk;
  fc->path = path;
  return fc;
}


void
FileCopyList_Add(FileCopyList fcl,
                 FileCopy fc) {
  ASH_PutMapping(fcl, fc->disk->id, fc);
}


void
FileCopyList_Free(FileCopyList fcl,
                  int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(fcl, it)
      FileCopy_Free((FileCopy)ASH_IteratorValue(it));
  ASH_HashTableFree(fcl);
}


/*** FILE TRANSFER Functions ***/


void
FileTransfer_Free(FileTransfer ft) {
  if(ft->id != NULL)
    FREE(ft->id);
  FREE(ft);
}


FileTransfer
FileTransfer_New(File file,
                 Disk source,
                 Disk dest) {
  FileTransfer ft = (FileTransfer)MALLOC(sizeof(struct _FileTransfer));
  ft->id =
    ASSTR_StrAppend(NULL, file->name, " ", source->id, " ", dest->id, NULL);
  ft->file = file;
  ft->source = source;
  ft->dest = dest;
  ft->stat = ACTION_NEW;
  memset(&ft->meta, 0, sizeof(ft->meta));
  return ft;
}


void
FileTransferList_Add(FileTransferList ftl,
                     FileTransfer ft) {
  ASH_PutMapping(ftl, ft->id, ft);
}


void
FileTransferList_Free(FileTransferList ftl,
                      int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(ftl, it)
      FileTransfer_Free((FileTransfer)ASH_IteratorValue(it));
  ASH_HashTableFree(ftl);
}


/*** GRIDINFOSOURCE Functions ***/

void
GridInfoSource_Free(GridInfoSource gis) {
  if(gis->id != NULL)
    FREE(gis->id);
  if(gis->server != NULL)
    FREE(gis->server);
}

GridInfoSource
GridInfoSource_New(char *id,
                   ELMETRIC_MetricServiceTypes method,
                   char *server) {
  GridInfoSource result =
    (GridInfoSource)MALLOC(sizeof(struct _GridInfoSource));
  result->id = id;
  result->method = method;
  result->server = server;
  return result;
}

void
GridInfoSourceList_Add(GridInfoSourceList gisl,
                       GridInfoSource gis) {
  ASH_PutMapping(gisl, gis->id, gis);
}

void
GridInfoSourceList_Free(GridInfoSourceList gisl,
                        int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(gisl, it)
      GridInfoSource_Free((GridInfoSource)ASH_IteratorValue(it));
  ASH_HashTableFree(gisl);
}


/*** HOST Functions ***/


void
Host_Free(Host h) {
  HostScheduler next;
  HostScheduler s;
  if(h->id != NULL)
    FREE(h->id);
  if(h->dnsName != NULL)
    FREE(h->dnsName);
  if(h->wd != NULL)
    FREE(h->wd);
  if(h->server != NULL)
    FREE(h->server);
  if(h->tasksCompleted != NULL)
    VectorFree(h->tasksCompleted);
  if(h->tasksInProgress != NULL)
    VectorFree(h->tasksInProgress);
  for(s = h->schedulers; s != NULL; s = next) {
    next = s->next;
    if(s->attrs != NULL)
      StrArrayFree(s->attrs);
    FREE(s);
  }
  FREE(h);
}


Host
Host_New(char *id,
         Disk disk,
         char *wd,
         ELBASE_ComputeServiceTypes method,
         char *server) {
  Host host = (Host)MALLOC(sizeof(struct _Host));
  host->id = id;
  host->disk = disk;
  host->dnsName = NULL;
  host->wd = wd;
  host->method = method;
  host->server = server;
  host->cpus = 1;
  host->memory = 0;
  host->tasksCompleted = VectorNew(Task);
  SetIncrement(host->tasksCompleted, 10);
  host->tasksInProgress = VectorNew(Task);
  host->schedulers = NULL;
  memset(&host->meta, 0, sizeof(host->meta));
  host->meta.timeDisabled = 0;
  return host;
}


void
HostList_Add(HostList hl,
             Host h) {
  ASH_PutMapping(hl, h->id, h);
}


void
HostList_Free(HostList hl,
              int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(hl, it)
      Host_Free((Host)ASH_IteratorValue(it));
  ASH_HashTableFree(hl);
}


/*** TASK Functions ***/


void
SortTasks(Task *tasks,
          unsigned len,
          int byPriority) {
  unsigned half = len / 2;
  unsigned i;
  unsigned j;
  Task t;
  if(len <= 1)
    return;
  SortTasks(tasks, half, byPriority);
  SortTasks(tasks + half, len - half, byPriority);
  for(i = 0, j = half; i < j && j < len; i++) {
    long left = byPriority ? -(long)tasks[i]->priority : (long)tasks[i]->seq;
    long right = byPriority ? -(long)tasks[j]->priority : (long)tasks[j]->seq;
    if(left <= right)
      continue;
    t = tasks[j];
    memmove(tasks + i + 1, tasks + i, (j - i) * sizeof(Task));
    tasks[i] = t;
    j++;
  }
}


void
Task_Free(Task t) {
  if(t->arguments != NULL)
    StrArrayFree(t->arguments);
  if(t->input != NULL)
    VectorFree(t->input);
  if(t->output != NULL)
    VectorFree(t->output);
  FREE(t);
}


Task
Task_New(void) {
  static SequenceNumber nextSeq = 0;
  Task t = (Task)MALLOC(sizeof(struct _Task));
  t->seq = nextSeq++;
  t->priority = 0;
  t->exec = NULL;
  t->arguments = NULL;
  t->input = VectorNew(File);
  t->output = VectorNew(File);
  t->stat = ACTION_NEW;
  t->host = NULL;
  memset(&t->meta, 0, sizeof(t->meta));
  return t;
}


void
TaskList_Add(TaskList tl,
             Task t) {
  ASH_PutMapping(tl, (ASH_KeyType)t->seq, t);
}


void
TaskList_Free(TaskList tl,
              int freeElements) {
  ASH_HashIterator it;
  if(freeElements)
    ASH_ForEachMapping(tl, it)
      Task_Free((Task)ASH_IteratorValue(it));
  ASH_HashTableFree(tl);
}
