/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include "ghs.h"

//#define PORT 3490 // the port client will be connecting to 

#define MAX_DATA_SIZE 100 // max number of bytes we can get at once
#define MAX_PARA_SIZE 120
#define BACKLOG 10     // how many pending connections queue will hold

void 
SigchldHandler(int s)
{
    while(wait(NULL) > 0);
}

float
GetMachineSpeed()
{

  FILE *fdConfig;
  char bufConfig[100];
  char ghsConfig[100];
  //This array is used to store the configuration parameters 
  char configArray[100][100];
  char *temp, *temp1;
  int i = 0, j = 0;
  float mSpeed;

  sprintf(ghsConfig, "%s%s", getenv("GHS_HOME"), "/ghs_config.dat");
  if (COMM_DEBUG_ON == 1) printf(" I get the ghs_config: %s \n", ghsConfig);
  if ( (fdConfig = fopen(ghsConfig, "r") ) != NULL )
  {
    while (fgets(bufConfig, 100, fdConfig) != NULL)
    {
      if ((bufConfig[0] == 47)&&(bufConfig[1] == 47))
      { /*Process the comments here */
      }
      else {
        strcpy(configArray[i], bufConfig);
        i++; // number of configuration line
      }
    }
  }
  else {
    printf("\nCannot open the configuration file");
  }

  for(j = 0; j < i; j++)
  {
    temp = strtok(configArray[j]," ");
    temp1 = strtok(NULL," ");
    if (strcmp(temp,"MACHINE_SPEED") == 0) // unit_time, usually it is an hour
    {
      sscanf(temp1, "%f ", &mSpeed);
      if (COMM_DEBUG_ON == 1) printf("\n the machine speed is: %f \n", mSpeed);
    }

  }

  fclose(fdConfig);
  return (mSpeed);
}

int 
GetParameters(char *paraValues, int period)
{
  char cmdLine[80];
  char buf[BUF_SIZE];
  char *predCmd = "syspred ";
  float utilization, jobArrival, mSpeed, servstd;
  FILE *ptr;

  sprintf(cmdLine, "%s/bin/%s utilization %d ", getenv("GHS_HOME"), predCmd, period);
  if ((ptr = popen(cmdLine, "r")) != NULL)
  {
    if (fgets(buf, BUF_SIZE, ptr) != NULL)
    {
      sscanf(buf, "%f", &utilization);
    }
    else
    {
      printf("can not get utilization \n");
      exit(1);
    }
    pclose(ptr);
  }

  sprintf(cmdLine, "%s arrival %d ", predCmd, period);
  if ((ptr = popen(cmdLine, "r")) != NULL)
  {
    if (fgets(buf, BUF_SIZE, ptr) != NULL)
    {
      sscanf(buf, "%f", &jobArrival);
    }
    else
    {
      printf("can not get job arrival rate \n");
      exit(1);
    }
    pclose(ptr);
  }

  sprintf(cmdLine, "%s std %d ", predCmd, period);
  if ((ptr = popen(cmdLine, "r")) != NULL)
  {
    if (fgets(buf, BUF_SIZE, ptr) != NULL)
    {
      sscanf(buf, "%f", &servstd);
    }
    else
    {
      printf("can not get utilization \n");
      exit(1);
    }
    pclose(ptr);
  }

  mSpeed = GetMachineSpeed();

  sprintf(paraValues, "%f %f %f %f", utilization/100.0, jobArrival, servstd, mSpeed);
  return(1);
}

int 
main(int argc, char *argv[])
{
  int newfd, sockDaemonfd, numBytes;
  int i = 0;
  char buf[MAX_DATA_SIZE];
  char paraBuf[MAX_PARA_SIZE];
  char hostName[80];
  int hn, hostNamelength = 80;
  struct hostent *he;
  struct sockaddr_in myAddr;    // my address information
  struct sockaddr_in serverAddr; // GHS server's address information
  int sinSize;
  struct sigaction sa;
  int yes = 1;
  int predScope = 1;
  int messageType;

  if ((sockDaemonfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
  {
    perror("socket");
    exit(1);
  }

  if (setsockopt(sockDaemonfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
  {
    perror("setsockopt");
    exit(1);
  }
    
  myAddr.sin_family = AF_INET;         // host byte order
  myAddr.sin_port = htons(DAEMON_PORT);     // short, network byte order
  myAddr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
  memset(&(myAddr.sin_zero), '\0', 8); // zero the rest of the struct

  if (bind(sockDaemonfd, (struct sockaddr *)&myAddr, sizeof(struct sockaddr)) == -1) 
  {
    perror("bind");
    exit(1);
  }

  if (listen(sockDaemonfd, BACKLOG) == -1) 
  {
    perror("listen");
    exit(1);
  }

  sa.sa_handler = SigchldHandler; // reap all dead processes
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = SA_RESTART;
  if (sigaction(SIGCHLD, &sa, NULL) == -1)
  {
    perror("sigaction");
    exit(1);
  }

  while (1)
  {
    sinSize = sizeof(struct sockaddr_in);
    
    if ((newfd = accept(sockDaemonfd, (struct sockaddr *)&serverAddr, &sinSize)) == -1)
    {
      perror("accept");
      continue;
    }

    printf("GHS daemon: got connection from %s\n", inet_ntoa(serverAddr.sin_addr));
    
    if (!fork()) // this is the child process
    { 
      close(sockDaemonfd); // child doesn't need the listener

      if ((numBytes = recv(newfd, buf, MAX_DATA_SIZE - 1, 0)) == -1) 
      {
        perror("recv");
        exit(1);
      }

      buf[numBytes] = '\0';

      if (COMM_DEBUG_ON == 1) printf("Received: %s", buf);
      
      sscanf(buf, "%d ", &messageType);
      if (messageType == CLIENT_INQUIRY)
      {
        sscanf(buf, "%d %d", &messageType, &predScope);
        GetParameters(paraBuf, predScope);
        if (COMM_DEBUG_ON == 1) printf("the parameter values are %s \n", paraBuf);
        
        sprintf(buf, "%d %s \n", CLIENT_REPLY, paraBuf);
        if (send(newfd, buf, MAX_DATA_SIZE, 0) == -1) perror("send");
        if (COMM_DEBUG_ON == 1) printf("I have sent the buf %s \n", buf);
      }
	  
      close(newfd);

      //UpdateParaFile(argv[1], buf);

      exit(0);
    }

    close(newfd);
    
  }

  return 0;
}

