/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/wait.h>
#include <signal.h>
#include "ghs.h" 

//#define MYPORT 3490    // the port users will be connecting to

#define BACKLOG 10     // how many pending connections queue will hold

#define MAX_DATA_SIZE 100 // max number of bytes we can get at once

char paraBuf[MAX_DATA_SIZE];

void 
SigchldHandler(int s)
{
    while(wait(NULL) > 0);
}

void 
UpdateParaFile (char *fileName, char *paraValues)
{
  FILE *resourceFile;
  FILE *resourceFiletmp;
  char fname[80];
  char linetmp[MAX_DATA_SIZE];
  char machName[30], machNametmp[30];
  int matched = 0; // indicates whether this new machine is already added in the resource file
  char cpCmd[80];
  float util, arrival, service, servstd;
	
  sscanf(paraValues, "%s ", machNametmp);
  if (COMM_DEBUG_ON == 1) printf("machine name is %s \n", machNametmp);

  if ( (resourceFile = fopen(fileName, "r")) == NULL )
  {
    printf("can not open resource file, it is a new file \n");
    if ((resourceFile = fopen(fileName, "w")) == NULL)
    {
      printf("can not create resource file \n");
      exit(1);
    }
    fclose(resourceFile);
    resourceFile = fopen(fileName, "r");
  }

  sprintf(fname, "%s.tmp", fileName);

  if ( (resourceFiletmp = fopen(fname, "w")) == NULL)
  {
    printf("can not open resource tmp file \n");
    exit(1);
  }

  while (fgets(linetmp, MAX_DATA_SIZE, resourceFile) != NULL)
  {
    sscanf(linetmp, "%s ", machName);
    if (COMM_DEBUG_ON == 1) printf("the line is %s ", linetmp);
    if (strcmp(machName, machNametmp) == 0)
    {
      fputs(paraValues, resourceFiletmp);
      matched = 1;
    }
    else
    {
      fputs(linetmp, resourceFiletmp);
    }
  }
	
  if (matched == 0) // it is a new machine information
  {
    fputs(paraValues, resourceFiletmp);
  }

  fclose(resourceFile);
  fclose(resourceFiletmp);

  sprintf(cpCmd, "cp %s %s ", fname, fileName);

  if (popen(cpCmd, "r") == NULL)
  {
    printf("can not execute cp command \n");
    exit(1);
  }
}

int 
//InquireParameters (char *paraValues, char *machName, int period)
InquireParameters (char *machName, int period)
{
  char buf[MAX_DATA_SIZE];
  char *str;
  int sockNodefd, numBytes;

  char hostName[80];
  int hn, hostNamelength = 80;
  struct hostent *he;
  struct sockaddr_in nodeAddr; // GHS daemon's address information
  int messageType;

  if ((he = gethostbyname(machName)) == NULL)
  {   
    perror("gethostbyname");
    exit(1);
  }

  if ((sockNodefd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
  {
    perror("socket");
    exit(1);
  }

  nodeAddr.sin_family = AF_INET;    // host byte order 
  nodeAddr.sin_port = htons(DAEMON_PORT);  // short, network byte order 
  nodeAddr.sin_addr = *((struct in_addr *)he->h_addr);
  memset(&(nodeAddr.sin_zero), '\0', 8);  // zero the rest of the struct 

  if (connect(sockNodefd, (struct sockaddr *)&nodeAddr, sizeof(struct sockaddr)) == -1) 
  {
    perror("connect");
    exit(1);
  }

  sprintf(buf, "%d %d \n", CLIENT_INQUIRY, period);
  if (send(sockNodefd, buf, MAX_DATA_SIZE, 0) == -1) perror("send");
  
  if ((numBytes = recv(sockNodefd, buf, MAX_DATA_SIZE - 1, 0)) == -1) 
  {
    perror("recv");
    exit(1);
  }

  if (COMM_DEBUG_ON == 1) printf("I have received the buf %s \n", buf);
  str = strtok(buf, " ");
  str = strtok(NULL, "\n");
  strcpy(paraBuf, str);
  //sscanf(buf, "%d %s", &messageType, &paraBuf);
  if (COMM_DEBUG_ON == 1) printf("the paravalues is %s \n", paraBuf);

  close(sockNodefd);

  return(1);
}

int 
main(int argc, char *argv[])
{
  int sockServerfd, newfd;  // listen on sock_fd, new connection on newfd
  int numBytes;
  char buf[MAX_DATA_SIZE];
  //char paraBuf[MAX_DATA_SIZE];
  struct sockaddr_in myAddr;    // my address information
  struct sockaddr_in theirAddr; // connector's address information
  int sinSize;
  struct sigaction sa;
  int yes = 1;
  int predScope = 1;
  int messageType;
  char machName[40];

  /* if (argc != 2) 
  {
    fprintf(stderr,"usage: server resource-file-name\n");
    exit(1);
  } */

  if ((sockServerfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
  {
    perror("socket");
    exit(1);
  }

  if (setsockopt(sockServerfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) == -1)
  {
    perror("setsockopt");
    exit(1);
  }
    
  myAddr.sin_family = AF_INET;         // host byte order
  myAddr.sin_port = htons(SERVER_PORT);     // short, network byte order
  myAddr.sin_addr.s_addr = INADDR_ANY; // automatically fill with my IP
  memset(&(myAddr.sin_zero), '\0', 8); // zero the rest of the struct

  if (bind(sockServerfd, (struct sockaddr *)&myAddr, sizeof(struct sockaddr)) == -1) 
  {
    perror("bind");
    exit(1);
  }

  if (listen(sockServerfd, BACKLOG) == -1) 
  {
    perror("listen");
    exit(1);
  }

  sa.sa_handler = SigchldHandler; // reap all dead processes
  sigemptyset(&sa.sa_mask);
  sa.sa_flags = SA_RESTART;
  if (sigaction(SIGCHLD, &sa, NULL) == -1)
  {
    perror("sigaction");
    exit(1);
  }

  while(1) // main accept() loop
  {  
    sinSize = sizeof(struct sockaddr_in);
    if ((newfd = accept(sockServerfd, (struct sockaddr *)&theirAddr, &sinSize)) == -1)
    {
      perror("accept");
      continue;
    }
    
    printf("GHS server: got connection from %s\n", inet_ntoa(theirAddr.sin_addr));
    if (!fork()) // this is the child process
    { 
      close(sockServerfd); // child doesn't need the listener
      /* if (send(newfd, "Hello, world!\n", 14, 0) == -1)
      perror("send");
      close(newfd); */
		
      if ((numBytes = recv(newfd, buf, MAX_DATA_SIZE - 1, 0)) == -1) 
      {
        perror("recv");
        exit(1);
      }

      buf[numBytes] = '\0';

      if (COMM_DEBUG_ON == 1) printf("Received: %s", buf);
      
      sscanf(buf, "%d ", &messageType);
      if (messageType == SERVER_INQUIRY)
      {
        sscanf(buf, "%d %s %d", &messageType, &machName, &predScope);
        InquireParameters(machName, predScope);
        
        sprintf(buf, "%d %s \n", SERVER_REPLY, paraBuf);
        if (send(newfd, buf, MAX_DATA_SIZE, 0) == -1) perror("send");
        if (COMM_DEBUG_ON == 1) printf("I have send the buf is %s \n", buf);
      }
	  
      close(newfd);

      //UpdateParaFile(argv[1], buf);

      exit(0);
    }
    close(newfd);  // parent doesn't need this
  }

  return 0;
}
