/* $Id: gantt.h,v 1.7 2003/07/25 18:15:34 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef GANTT_H
#define GANTT_H


#ifdef __cplusplus
extern "C" {
#endif


/* A client-specified type for events.  Type -1 is reserved. */
typedef int GanttEventType;


typedef struct _GanttEvent {
  GanttEventType type;
  unsigned long begin;
  unsigned long end;
  void *details;
  struct _GanttEvent *next;
  struct _GanttEvent *previous;
} *GanttEvent;

typedef struct _GanttColumn {
  void *key;
  GanttEvent firstEvent;
  GanttEvent lastEvent;
  GanttEvent firstHole;
} *GanttColumn;

typedef struct _GanttChart {
  GanttColumn *columns;
} *GanttChart;


/* Frees all memory allocated for #e#. */
void
GanttEvent_Free(GanttEvent e);


/* Creates a new GanttEvent with the specified characteristics. */
GanttEvent
GanttEvent_New(int type,
               unsigned long begin,
               unsigned long end,
               void *details);


/* Adds #e# to #c#. */
void
GanttColumn_AddEvent(GanttColumn c,
                     GanttEvent e);


/*
 * Returns the earliest time a #runTime#-long event can begin if entered in
 * #c# at or after #earliestStart#.
 */
unsigned long
GanttColumn_EarliestStartTime(GanttColumn c,
                              unsigned long earliestStart,
                              unsigned long runTime);


/* Returns the number of events entered in #c#. */
unsigned long
GanttColumn_GetEventCount(GanttColumn c);


/* Returns the event from #c# for #details#, NULL if none. */
GanttEvent
GanttColumn_GetEventForDetails(GanttColumn c,
                               void *details);


/* Returns the first event entered in #c#, NULL if none. */
GanttEvent
GanttColumn_GetFirstEvent(GanttColumn c);
#define GanttColumn_IsEmpty(c) (GanttColumn_GetFirstEvent(c) == 0)


/* Returns the key of #c#. */
void *
GanttColumn_GetKey(GanttColumn c);


/* Returns the event that follows #e# in #c#, NULL if none. */
GanttEvent
GanttColumn_GetNextEvent(GanttColumn c,
                         GanttEvent e);


/* Removes #e# from #c#. Returns #e# for convenience. */
GanttEvent
GanttColumn_RemoveEvent(GanttColumn c,
                        GanttEvent e);


/* Adds a new column with key #key# to #g# and returns the new column. */
GanttColumn
GanttChart_AddColumn(GanttChart g,
                     void *key);


/* Frees all memory allocated for #g#. */
void
GanttChart_Free(GanttChart g);


/* Returns the #n#th column (starting at 0) of #g#, NULL if none. */
GanttColumn
GanttChart_GetColumn(GanttChart g,
                     unsigned n);


/* Returns the number of columns in #g#. */
int
GanttChart_GetColumnCount(GanttChart g);


/* Returns the column in #g# containing an event for #details#, NULL if none. */
GanttColumn
GanttChart_GetColumnForDetails(GanttChart g,
                               void *details);


/* Returns the column with key #key# from #g#, NULL if no such column exits. */
GanttColumn
GanttChart_GetColumnForKey(GanttChart g,
                           void *key);


/* Returns 1 iff every column in #g# is empty, else 0. */
int
GanttChart_IsEmpty(GanttChart g);


/* Returns a new, empty GanttChart. */
GanttChart
GanttChart_New(void);


#define GanttColumn_ForEachEvent(c,e) \
  for(e=GanttColumn_GetFirstEvent(c); e!=0; e=GanttColumn_GetNextEvent(c,e))


#ifdef __cplusplus
}
#endif


#endif
