/* $Id: global.h,v 1.53 2003/12/01 19:07:34 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef GLOBAL_H
#define GLOBAL_H


#include <stdio.h> /* FILE */
#include <diagnosticseed.h>
#include <xmlseed.h>
#include "corestructures.h"

/* for GHS -START*/
typedef enum {
  HEURISTIC_ROUNDROBIN,
  HEURISTIC_MINMIN,
  HEURISTIC_MAXMIN,
  HEURISTIC_SUFFERAGE,
  HEURISTIC_WORK_QUEUE,
  HEURISTIC_XSUFFERAGE,
  NB_HEURISTICS,
  HEURISTIC_GHS
} heuristic_t;
/* for GHS -END*/

extern struct Global {

  DiskList disks;                 /* All defined disks. */
  FileList files;                 /* All known files. */
  GridInfoSourceList gridInfoSources; /* All known info sources. */
  HostList hosts;                 /* All defined hosts. */
  TaskList tasks;                 /* All defined tasks. */
  FileTransferList transfers;         /* All file transfers. */
  FileTransferList transfersCanceled; /* Subsets canceled, done, etc. */
  FileTransferList transfersDone;
  FileTransferList transfersFailed;
  FileTransferList transfersInProgress;

  /* Named task groups */
  ASH_HashTable taskGroups;

  /*
   * Auxilliary task lists.  Since typically few tasks specify these options,
   * we can save considerable memory by offloading them into separate tables
   * rather than storing them in with each task.
   */
  ASH_HashTable stdins;  /* stdin files keyed by task position */
  ASH_HashTable stdouts; /* stdout files keyed by task position */
  ASH_HashTable stderrs; /* stderr files keyed by task position */
  ASH_HashTable userIds; /* user-defined task ids keyed by task position */
  ASH_HashTable wds;     /* working directories keyed by task position */

  int bulkCopy;
  FILE *checkpointFile;
  unsigned commandCount;
  int downloadstd;
  int findInput;
  heuristic_t heuristic;
  int simDisk;
  int simHost;
  int simRandom;
  unsigned long timeStarted;

} global;


extern ASDIAG_DiagnosticState ds;


/*
 * Builds the necessary pointers between #task# and #host#, as well as between
 * #task#'s files and #host#'s disks. This is a helper function that is called
 * only by sched_run() when it decides a scheduling assignment.
 */
void
AssignTaskToHost(Task t,
                 Host h);


/* Updates the global lists to show that #ft# has been canceled. */
void
FileTransferCanceled(FileTransfer ft);


/*
 * Updates the global lists to indicate that #ft# has failed or successfully
 * completed, depending on the value of #exitCode#.
 */
void
FileTransferDone(FileTransfer ft,
                 int exitCode);


/* Updates the global lists to show that #ft# has been reset. */
void
FileTransferReset(FileTransfer ft);


/* Updates the global lists to show that #ft# has been started. */
void
FileTransferStarted(FileTransfer ft);


/* Returns the local disk from the global disk list; creates it if necessary. */
Disk
FindLocalDisk(void);


/*
 * Returns allocated XML text that reflects the global state.  Only includes
 * task status information for those tasks with sequence numbers enumerated in
 * the #tasksCount#-long array #tasks#.
 */
char *
GetStateXML(SequenceNumber *tasks,
            unsigned tasksCount);


void
GlobalDeinit(void);


void
GlobalInit(void);


/*
 * Returns #section# XML for the elements of #hash# in an allocated string.
 * Passes each element of #hash# in turn to #fn# to get the element XML.
 */
typedef char *(*XmlFn)(ASH_ValueType);
char *
HashToXML(ASH_HashTable hash,
          const char *section,
          XmlFn fn);


/* Updates the global lists to indicate that #h# is disabled. */
void
HostDisabled(Host h);


/* Updates the global lists to indicate that #h# is enabled. */
void
HostEnabled(Host h);


/* Notes the production of a copy of #file# on #disk#. */
void
NewFileCopy(Disk disk,
            File file);


int
ProcessLogXml(ASXML_XmlNode node);


/* Updates the global lists to show that #t# has been canceled. */
void
TaskCanceled(Task t);


/*
 * Updates the global lists to indicate that #t# has failed or successfully
 * completed, depending on the value of #exitCode#.
 */
void
TaskDone(Task t,
         int exitCode);


/*
 * Updates the global lists to show the priority of #t# has been set to
 * #priority#.
 */
void
TaskPrioritized(Task t,
                unsigned priority);


/* Updates the global lists to show that #t# has been reset. */
void
TaskReset(Task t);


/* Updates the global lists to show that #t# has been started on #h#. */
void
TaskStarted(Task t,
            Host h);


void
TaskGroupAdd(const char *name,
             Task t);


unsigned long
TaskGroupMembership(const char *name);


void
TaskGroupRemove(const char *name,
                Task t);

             
#define ERROR(msg) ASDIAG_ERROR(ds,msg)
#define ERROR1(msg,a) ASDIAG_ERROR1(ds,msg,a)
#define ERROR2(msg,a,b) ASDIAG_ERROR2(ds,msg,a,b)
#define ERROR3(msg,a,b,c) ASDIAG_ERROR3(ds,msg,a,b,c)
#define ERROR4(msg,a,b,c,d) ASDIAG_ERROR4(ds,msg,a,b,c,d)
#define ERROR5(msg,a,b,c,d,e) ASDIAG_ERROR5(ds,msg,a,b,c,d,e)
#define LOG(msg) ASDIAG_LOG(ds,msg)
#define LOG1(msg,a) ASDIAG_LOG1(ds,msg,a)
#define LOG2(msg,a,b) ASDIAG_LOG2(ds,msg,a,b)
#define LOG3(msg,a,b,c) ASDIAG_LOG3(ds,msg,a,b,c)
#define LOG4(msg,a,b,c,d) ASDIAG_LOG4(ds,msg,a,b,c,d)
#define LOG5(msg,a,b,c,d,e) ASDIAG_LOG5(ds,msg,a,b,c,d,e)
#define FAIL(msg) ASDIAG_FAIL(ds,msg)
#define FAIL1(msg,a) ASDIAG_FAIL1(ds,msg,a)
#define FAIL2(msg,a,b) ASDIAG_FAIL2(ds,msg,a,b)
#define FAIL3(msg,a,b,c) ASDIAG_FAIL3(ds,msg,a,b,c)
#define FAIL4(msg,a,b,c,d) ASDIAG_FAIL4(ds,msg,a,b,c,d)
#define FAIL5(msg,a,b,c,d,e) ASDIAG_FAIL5(ds,msg,a,b,c,d,e)
#define FAILRETURN(msg,ret) ASDIAG_FAILRETURN(ds,msg,ret)
#define FAILRETURN1(msg,a,ret) ASDIAG_FAILRETURN1(ds,msg,a,ret)
#define FAILRETURN2(msg,a,b,ret) ASDIAG_FAILRETURN2(ds,msg,a,b,ret)
#define FAILRETURN3(msg,a,b,c,ret) ASDIAG_FAILRETURN3(ds,msg,a,b,c,ret)
#define FAILRETURN4(msg,a,b,c,d,ret) ASDIAG_FAILRETURN4(ds,msg,a,b,c,d,ret)
#define FAILRETURN5(msg,a,b,c,d,e,ret) ASDIAG_FAILRETURN5(ds,msg,a,b,c,d,e,ret)
#define WARN(msg) ASDIAG_WARN(ds,msg)
#define WARN1(msg,a) ASDIAG_WARN1(ds,msg,a)
#define WARN2(msg,a,b) ASDIAG_WARN2(ds,msg,a,b)
#define WARN3(msg,a,b,c) ASDIAG_WARN3(ds,msg,a,b,c)
#define WARN4(msg,a,b,c,d) ASDIAG_WARN4(ds,msg,a,b,c,d)
#define WARN5(msg,a,b,c,d,e) ASDIAG_WARN5(ds,msg,a,b,c,d,e)


#endif
