/* $Id: protocol.h,v 1.20 2003/05/08 15:48:04 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#ifndef APSTCLIENT_PROTOCOL_H
#define APSTCLIENT_PROTOCOL_H

/*
 * The APST daemon communicates with network clients using a simple, text-based
 * protocol.  Each message, both client requests and daemon replies, ends
 * with a CRLF (0x0A0C) pair.  Client requests begin with a command word.  Some
 * consist wholly of the command word; others follow the command word with
 * whitespace and content.  The format of individual commands is noted below.
 * The APST daemon responds to client requests by sending a reply that begins
 * with either "OK" (if the request was accepted and successfully executed) or
 * "FAIL" (if the request was rejected or could not be executed).  Each of
 * these words can be followed by whitespace and content.  For "FAIL" replies,
 * the content consists of a message explaining the failure; for "OK" replies,
 * the content consists of any information asked for by the client request.
 *
 * Each of the command descriptions below lists the form(s) of the command,
 * followed by the maximum security level under which the command will be
 * executed.  The security levels are NONE (all commands executed), MINIMAL
 * (harmless, state change, and information commands executed; ADD command
 * disallowed), MEDIUM (harmless and information commands executed) and STRICT
 * (only harmless commands executed).
 *
 * The CANCEL, RESET, and STATE commands are each followed by a task list.
 * This list consists of a space-separated set of task id ranges (id[-id])
 * and/or the words ALL, CANCELED, DONE, FAILED, NEW, and RUNNING.  The command
 * will affect only those tasks included in one of the elements of the list.
 * Each element may be preceded by a !, in which case the tasks specified by
 * that element will *not* be affected.
 */
#define APST_CLIENT_OK "OK"
#define APST_CLIENT_FAIL "FAIL"

/*
 * ADD xml
 * Security level: NONE
 * Add the resources and/or tasks detailed in #xml# to the daemon lists.  The
 * outermost element in #xml# must be a <apst> tag.  See resources.xml and/or
 * the apstd man page for a description of the allowed subtags.  A successful
 * reply contains no content.
 */
#define APST_CLIENT_ADD "ADD"

/*
 * CANCEL list
 * Security level: MINIMAL
 * Cancel any uncompleted tasks included in #list#.  A successful reply
 * contains no content.
 */
#define APST_CLIENT_CANCEL_TASKS "CANCEL"

/*
 * CLEAR
 * Security level: MINIMAL
 * Cancel all running tasks and discard all task and resource information.
 * A successful reply carries no content.
 */
#define APST_CLIENT_CLEAR "CLEAR"

/*
 * DISABLE ALL
 * DISABLE host
 * Security level: MINIMAL
 * Mark as unusable all hosts (DISABLE ALL) or the listed #host#.  A successful
 * reply contains no content.
 */
#define APST_CLIENT_DISABLE_HOSTS "DISABLE"

/*
 * DOWNLOAD file
 * Security level: MINIMAL
 * Copy the output file #file# from a remote disk to the local disk.  A
 * successful reply contains no data.
 */
#define APST_CLIENT_DOWNLOAD_FILE "DOWNLOAD"

/*
 * ENABLE ALL
 * ENABLE host
 * Security level: MINIMAL
 * Mark as usable all hosts (ENABLE ALL) or the listed #host#.  A successful
 * reply contains no content.
 */
#define APST_CLIENT_ENABLE_HOSTS "ENABLE"

/*
 * FILE
 * Security level: MEDIUM
 * Retrieve the daemon log file.  A successful reply contains the file contents.
 */
#define APST_CLIENT_GET_FILE "FILE"

/*
 * ID
 * Security level: STRICT
 * Retrieve the client id of the sender.  The daemon assigns a unique id to
 * each client; this command retrieves it.  A successful reply contains the
 * sender id and the daemon working directory as content.
 */
#define APST_CLIENT_GET_ID "ID"

/*
 * NOOP
 * Security level: STRICT
 * Do nothing; used for testing.  A successful reply contains no content.
 */
#define APST_CLIENT_NOOP "NOOP"

/*
 * NOTIFY email
 * Security level: MINIMAL
 * Add #email# to the list of addresses to be contacted when all tasks are
 * completed.  A successful reply contains no content.
 */
#define APST_CLIENT_NOTIFY "NOTIFY"

/*
 * PRIORITIZE int list
 * Security level: MINIMAL
 * Change the priority of the tasks in #list# to #int# and reschedule.
 */
#define APST_CLIENT_PRIORITIZE "PRIORITIZE"

/*
 * READ file
 * Security level: STRICT
 * READ a command from #file#, which must be in the daemon working directory.
 * Commands read from files have no security restrictions; the daemon will
 * execute any command, no matter what its security level.  A successful reply
 * contains the content, if any, of the command in #file#.
 */
#define APST_CLIENT_READ "READ"

/*
 * RESET list
 * Security level: MINIMAL
 * Mark all tasks included in #list# as incomplete. A successful reply contains
 * no content.
 */
#define APST_CLIENT_RESET_TASKS "RESET"

/*
 * STATE list
 * Security level: MEDIUM
 * Retrieve xml that represents the current state of the execution.  A
 * successful reply returns xml with the same format as that passed to the ADD
 * command.  This xml details all known resources and all tasks included in
 * #list#.
 */
#define APST_CLIENT_GET_STATE "STATE"

/*
 * STATUS
 * Security level: MEDIUM
 * Retrieve the computation status.  A successful reply contains seven space-
 * separated integers: 1) the number of seconds the daemon has been running;
 * 2) the number of commands performed; 3) total tasks; 4) tasks left to do;
 * 5) tasks running; 6) tasks finished; 7) tasks failed.
 */
#define APST_CLIENT_GET_STATUS "STATUS"

/*
 * STEP n
 * Security level: MINIMAL
 * Only execute #n# more scheduling steps (a launch of an event on each idle
 * resource) before pausing for a subsequent STEP command.  A negative #n#
 * allows the daemon to run without pause.
 */
#define APST_CLIENT_STEP "STEP"

/*
 * STOP
 * Security level: MINIMAL
 * Terminate the daemon.  A successful reply contains no content.
 */
#define APST_CLIENT_STOP "STOP"


#endif
