/* $Id: misc.c,v 1.40 2003/09/17 18:14:41 jhayes Exp $ */
/*
 * Copyright  2002 The Regents of the University of California. 
 * All Rights Reserved. 
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for educational, research and non-profit purposes, without
 * fee, and without a written agreement is hereby granted, provided that the
 * above copyright notice, this paragraph and the following three paragraphs
 * appear in all copies. 
 *
 * Permission to incorporate this software into commercial products may be
 * obtained by contacting
 * Technology Transfer Office 
 * 9500 Gilman Drive 
 * 411 University Center 
 * University of California 
 * La Jolla, CA 92093-0093
 * (858) 534-5815
 * invent@ucsd.edu
 *
 * This software program and documentation are copyrighted by The Regents of
 * the University of California. The software program and documentation are
 * supplied "as is", without any accompanying services from The Regents. The
 * Regents does not warrant that the operation of the program will be
 * uninterrupted or error-free. The end-user understands that the program was
 * developed for research purposes and is advised not to rely exclusively on
 * the program for any reason. 
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING
 * LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION,
 * EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED
 * HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO
 * OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR
 * MODIFICATIONS. 
 */


#include "config.h"
#include <sys/types.h>
#include <ctype.h>    /* isdigit isupper */
#include <stdlib.h>   /* free malloc realloc */
#include <stdio.h>    /* file functions */
#include <string.h>   /* string functions */
#include <unistd.h>   /* dup dup2 getpid unlink */
#include <fileseed.h>
#define ASIP_SHORT_NAMES
#include <ipseed.h>
#define ASSTR_SHORT_NAMES
#include <strseed.h>
#include <vectorseed.h>
#include "misc.h"


static const char *CRLF = "\015\012";
static char **emails = NULL;


char *
AbsolutePath(const char *path,
             const char *dir) {
  return *path == '/' || *path == '$' ?
         STRDUP(path) : StrAppend(NULL, dir, "/", path, NULL);
}


void
AddEmailNotification(const char *address) {
  if(emails == NULL)
    emails = ASV_VectorNew(char *);
  ASV_Append(emails, STRDUP(address));
}


unsigned
SendEmailNotifications(const char *subject,
                       const char *body) {
  char command[256];
  unsigned i;
  FILE *pipe;
  if(emails == NULL)
    return 0;
  for(i = 0; i < ASV_Size(emails); i++) {
    sprintf(command, "mail -s \"%s\" %s", subject, emails[i]);
    if((pipe = popen(command, "w")) == NULL)
      continue;
    fprintf(pipe, "%s", body);
    fclose(pipe);
  }
  return ASV_Size(emails);
}


void
my_free(void *ptr) {
  free(ptr);
}


void *
my_malloc(size_t size) {
  void *ptr = malloc(size);
  return ptr;
}


void *
my_realloc(void *ptr,
           size_t size) {
  char *newptr = ptr == NULL ? malloc(size) : realloc(ptr, size);
  return newptr;
}


char *
my_strdup(const char *s) {
  char *ptr = strdup(s);
  return ptr;
}


void
StrIncr(char **s,
        unsigned amount) {
  unsigned base = 10;
  char *c;
  char *extended;
  unsigned incr;
  char upperBound = '9';
  for(c = *s + strlen(*s) - 1; amount > 0; c--) {
    if(c < *s) {
      extended = StrAppend(NULL, " ", *s, NULL);
      *extended = upperBound - base + 1;
      FREE(*s);
      c = *s = extended;
    }
    base = isdigit((int)*c) ? 10 : 26;
    upperBound = isdigit((int)*c) ? '9' : isupper((int)*c) ? 'Z' : 'z';
    incr = amount % base;
    amount /= base;
    if((*c += incr) > upperBound) {
      *c -= base;
      amount++;
    }
  }
}


int
ReceiveString(Socket sock,
              char **value) {
  char *c;
  char lf;
  if(!ReceiveStringTerminated(sock, CRLF, value) || !Receive(sock, &lf, 1))
    return 0;
  if((c = strchr(*value, *CRLF)) != NULL)
    *c = '\0';
  return 1;
}


int
SendString(Socket sock,
           const char *value) {
  char *c;
  char *crless;
  int result;
  if(strchr(value, *CRLF) == NULL)
    return Send(sock, value, strlen(value)) && Send(sock, CRLF, strlen(CRLF));
  /*
   * A CR in #value# will confuse the receiving code (see above), which treats
   * CR as the message terminator and assumes that LF follows immediately.  The
   * proper thing to do would be to en/decode CRs.  However, in practice the
   * only time this occurs is in the line terminators of files (e.g. XML) that
   * were generated on Windows systems.  The receiver will ignore these CRs, so
   * it's easiest just to replace them with spaces before sending.
   */
  crless = strdup(value);
  for(c = strchr(crless, *CRLF); c != NULL; c = strchr(c + 1, *CRLF))
    *c = ' ';
  result = Send(sock, crless, strlen(crless)) && Send(sock, CRLF, strlen(CRLF));
  free(crless);
  return result;
}


#define STDERR_CAPTURE_PATH_FORMAT  "/tmp/apstd_stderr_%d_%d"


int
StderrCaptureBegin(void) {
  FILE *f;
  int key = dup(2);
  char path[80];
  sprintf(path, STDERR_CAPTURE_PATH_FORMAT, (int)getpid(), key);
  if((f = fopen(path, "w")) != NULL) {
    dup2(fileno(f), 2);
    fclose(f);
  }
  return key;
}


void
StderrCaptureEnd(int key) {
  dup2(key, 2);
}


char *
StderrCaptureRetrieve(int key) {
  char path[80];
  char *result;
  sprintf(path, STDERR_CAPTURE_PATH_FORMAT, (int)getpid(), key);
  result = ASFILE_FileContents(path);
  unlink(path);
  close(key);
  return result;
}
