/*
 * Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS I release by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

#include<stdio.h>
#include<math.h>
#include<stdlib.h>
#include<time.h>
#include "ghs.h"

double maxJobWorkload = 2000;

double utilPara[MAX_WS_NUM], servPara[MAX_WS_NUM], servtmp[MAX_WS_NUM], arriPara[MAX_WS_NUM], servSTDPara[MAX_WS_NUM], workPara[MAX_WS_NUM], speedPara[MAX_WS_NUM];

char machName[MAX_WS_NUM][30];
char taskName[MAX_WS_NUM][30];

int maptab[MAX_WS_NUM][MAX_GJOB_NUM];

double currworkload[MAX_GJOB_NUM];
int wsNum;
int jobNum;

// Grid Jobs workload array
double gJobWorkload[MAX_GJOB_NUM];
int lastItemIndex[MAX_WS_NUM];


/* task partition for a group of independ tasks
 * each task has its own workload and can not be further partitioned
 * we use the mean-mean partition method
 */

void 
PartitionWorkload()
{
  int i = 0, j = 0, k = 0;
  int bestJob, bestMachine, tmpMachine;
  int jobtag[MAX_GJOB_NUM];
  int firstJobCond;
  int firstMachCond;

  // the sum of (1-utilization)* speed
  double globalBestTime, tmpBestTime, finishTime;

  for (i = 0; i < jobNum; i++) 
  {
    jobtag[i] = 0;
  }

  for (i = 0; i < wsNum; i++) 
  {
    lastItemIndex[i] = 0;
    currworkload[i] = 0;
  }

  for (i = 0; i < jobNum; i++) 
  {
    firstJobCond = 0;

    for (k = 0;k < jobNum; k++)
    {

      if ( jobtag[k] == 0) 
      {
			  
        firstMachCond = 0;

        for (j = 0; j < wsNum; j++)
        {
          finishTime = (gJobWorkload[k] + currworkload[j]) / ((1 - arriPara[j] * servPara[j]) * speedPara[j]);
					
          if ( firstJobCond == 0) 
          {
            tmpBestTime = finishTime;
            globalBestTime = tmpBestTime;
            bestJob = k;
            bestMachine = j;
            firstJobCond = 1;
          }

          if (firstMachCond == 0) 
          {
            tmpBestTime = finishTime;
            tmpMachine = j;
            firstMachCond = 1;
          }

          if (tmpBestTime > finishTime) 
          {
            tmpBestTime = finishTime;
            tmpMachine = j;
          }
        } // end for
      } // end if

      if (globalBestTime > tmpBestTime) 
      {
        globalBestTime = tmpBestTime;
        bestJob = k;
        bestMachine = tmpMachine;
      }
    } // end for

    jobtag[bestJob] = 1;
    currworkload[bestMachine] += gJobWorkload[bestJob];
    maptab[bestMachine][lastItemIndex[bestMachine]] = bestJob;
    lastItemIndex[bestMachine]++;
  } // end for

  for (i = 0; i < wsNum; i++) 
  {
    workPara[i] = currworkload[i];
    //printf("\n the workload of machine %d is %f %f\n", i, workPara[i], workPara[i]/speedPara[i]);
  }

}


/*  read resource parameters
 *	resource name, utilization, arrival rate, servicestd
 */
void 
GetResourcePara(char *configFile) 
{
  int i = 0;
  FILE *stream;
  char line[100];
  float tmpUtil, tmpArri, tmpServ, tmpSpeed;

  if ( (stream = fopen(configFile, "r")) == NULL) 
  {
    printf("can't open the resource parameter file\n");
    exit(1);
  }

  //if( fgets( line, 100, stream ) != NULL) sscanf(line, "%d", &wsNum);

  while ( fgets(line, 100, stream) != NULL) 
  {
    sscanf(line, "%s %f %f %f %f", &machName[i], &tmpUtil, &tmpArri, &tmpServ, &tmpSpeed);
    utilPara[i] = tmpUtil;
    arriPara[i] = tmpArri;
    servSTDPara[i] = tmpServ;
    speedPara[i] = tmpSpeed;
    //printf(" %f %f %f %f \n", utilPara[i], arriPara[i], servSTDPara[i], speedPara[i]);
    i++;
  }

  wsNum = i;
  fclose(stream);

}

/*  read task parameters
 *	task name, task workload
 */

void 
GetTaskPara(char *configFile) 
{
  int i = 0;
  FILE *stream;
  char line[100];
  float tmpWork;

  if ( (stream = fopen(configFile, "r")) == NULL) 
  {
    printf("can't open the task parameter file\n");
    exit(1);
  }

  while (fgets(line, 100, stream) != NULL) 
  {
    sscanf(line, "%s %f", &taskName[i], &tmpWork);
    gJobWorkload[i] = tmpWork;
    i++;
  }

  jobNum = i;
  //printf("the workload is %s %f \n", taskName[0], tmpWork);
  fclose(stream);

}

/* write the task allocation result into a result file
 * which name is provided by argv[3]
 */

void 
OutputAllocation(char *outputFile)
{
  int i = 0, j;
  FILE *stream;
  char line[100];

  if ( (stream = fopen(outputFile, "w")) == NULL) 
  {
    printf("can't open output file\n");
    exit(1);
  }

  for (i = 0; i < wsNum; i++) 
  {
    sprintf(line, "%s : %f \n", machName[i], workPara[i]);
    fputs(line, stream);
    for (j = 0; j < lastItemIndex[i]; j++) 
    {
      sprintf(line, "   %s \n", taskName[maptab[i][j]]);
      fputs(line, stream);
    }
  }

  fclose(stream);

}

int 
main(int argc, char **argv)
{
  int i;

  if (argc != 4)
  {
    printf("mtal resopara.txt taskpara.txt result.txt \n");
    exit(1);
  }

  GetResourcePara(argv[1]);
  GetTaskPara(argv[2]);
  PartitionWorkload();
  OutputAllocation(argv[3]);

  return(0);

}
