/* Copyright (c) 2000-2003 Illinois Institute of Technology.
 *                         All rights reserved.
 *
 * This file is part of the GHS software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * GHS source distribution.
 *
 * Released Date: August 18 2005
 * This is the GHS 1.0 released by the
 *   SCS Group,
 *   http://www.cs.iit.edu/~scs
 *   Department of Computer Science,
 *   Illinois Institute of Technology.
 *
 * This release has been tested under SUN OS 5.9.
 *
 * Supervisor
 * ----------
 * Illiniois Institute of Technology:
 * -Dr. Xian-He Sun
 *
 * Author(s)
 * -----------------
 * Illinois Institute of Technology:
 * -Ming Wu
 * -Xian-He Sun
 *
 */

//#include "top.h"
//#include "machine.h"

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include "ghs.h"

#define MAX_DATA_SIZE 100 // max number of bytes we can get at once for one line
#define MAX_IT 5
#define MAX_SAMPLE_SIZE 150 // maximum sample size in an hour
#define MIN_SAMPLE_SIZE 10 // maximum sample size in an hour
#define INVALID_LAMBDA 111 // invalid job arrival rate, used as the bound of the valid lambda's array
#define INVALID_MU 11111 // invalid job service rate
#define INVALID_THETA 11111 // invalid theta 
#define INVALID_RHO 222 // invalid utilization, used as the bound of the valid utilization's array
#define MAX_PROC_NUM 1000 // maximum process number during a measuremnt time interval

#define data_log "data.log" // the log file storing the J_arrival and M_utilization original information

double procMiniTime = 1.0;

FILE *fdLog, *ptrtmp;
int stoi(char *str);
long FindTime(char *buf);
extern double CalculateMean(double *values,int num_elements);
extern double CalculateSTD(double *values,int num_elements);

/* This array is used to store the configuration parameters */
char configArray[100][100];

double lambdaMeanPredict[MAX_HOUR]; //J_Lambda expectation prediction array
double muMeanPredict[MAX_HOUR]; //M_Utilization expectation prediction array
double lambdaSTDPredict[MAX_HOUR]; //J_Lambda std prediction array
double muSTDPredict[MAX_HOUR]; //M_Utilization std prediction array

double lambdaValues[MAX_HOUR][MAX_SAMPLE_SIZE]; //J_Lambda sample values
double muValues[MAX_HOUR][MAX_SAMPLE_SIZE]; //J_serviceRate sample values
double thetaValues[MAX_HOUR][MAX_SAMPLE_SIZE]; //J_serviceSTD sample values
double rhoValues[MAX_HOUR][MAX_SAMPLE_SIZE]; //M_Utilization sample values

char nextSamples[5]; // string used to store the sample number in the next hour
char meanLambda[15]; // string used to store the expectation of J_lambda
char stdLambda[15]; // string used to store the std of J_lambda
char meanUtilization[15]; //string used to store the expectation of M_utilization
char stdUtilization[15]; //string used to store the std of M_utilization
char sampleSize[15];
char predict_result[50]; // string used to store the M_utilization results
char measureLog[100]; // string used to store the log file name
char meanMiu[15]; // string used to store the expectation of J_serviceRate
char meanTherta[15]; // string used to store the expectation of J_serviceSTD

char currHour[15]; // string used to store the current hour since UTC

int debugOn =0; // debug tag
int logfileRenew =0; // logfile renew tag
int lineNum = 1;

/* initial unit_time for measuring, the maximum measuring time_unit */
int sampleTime=3600, loopNum=1000;

/* time interval array */
int a[10000];

/* numIter means the number of samples during one hour,
 * numIterNext means the number of samples during the next hour
 */

int numIter = 0, numIterNext = 50;

int lastcommOn = 0;

int currSampleNum = 1; 

char startDate[40];

struct _JobStatistics sampleStatistics;
int measureInterval;

double jobServTime[MAX_PROC_NUM];

void
RemoveLines(char *fileName, int lineOfNum)
{
  FILE *resourceFile;
  FILE *resourceFiletmp;
  char fname[80];
  char linetmp[MAX_DATA_SIZE];
  int lineCount = 0;
  char cpCmd[80];

  if ( (resourceFile = fopen(fileName, "r")) == NULL )
  {
    printf("can not open resource file to delete lines \n");
    exit(1);
  }

  sprintf(fname, "%s.ttmp", fileName);

  if ( (resourceFiletmp = fopen(fname, "w")) == NULL)
  {
    printf("can not open resource tmp file \n");
    exit(1);
  }

  lineCount = 0;
  while (fgets(linetmp, MAX_DATA_SIZE, resourceFile) != NULL)
  {
    if (DEBUG_ON == 1) printf("the line is %s ", linetmp);
    if ( (lineCount > lineOfNum) || (lineCount == 0) ) // leave the first line there
    {
      fputs(linetmp, resourceFiletmp);
    };
    lineCount++;
  }

  fclose(resourceFile);
  fclose(resourceFiletmp);

  sprintf(cpCmd, "cp %s %s ", fname, fileName);

  if (popen(cpCmd, "r") == NULL)
  {
    printf("can not execute cp command \n");
    exit(1);
  }
}

int
GetLineNum(char *fileName)
{
  FILE *resourceFile;
  char linetmp[MAX_DATA_SIZE];
  int lineCount = 0;

  if ( (resourceFile = fopen(fileName, "r")) == NULL )
  {
    printf("can not open file to read \n");
    exit(1);
  }

  lineCount = 0;
  while (fgets(linetmp, MAX_DATA_SIZE, resourceFile) != NULL)
  {
    lineCount++;
  }

  fclose(resourceFile);

  return lineCount;
}

/* This function converts string to integer
 * same as stoi 
 */

int 
stoi(char *str)
{
  int len, i = 0, num = 0;
  int mul = 1;
        
  len = strlen(str);

  for(i = len-1; i >= 0; i--)
  {
    num += (str[i] - 48) * mul;
    mul *= 10;
  }
  
  return num;
}       


/* This function reads the time from the date command in 
 *	HH:MM:SS format and converts into total number of seconds   by multiplying 
 *	HH * 3600 + MM * 60 + ss 
 */
long 
FindTime(char *buf)
{
  char *temp, *temp1;
  long time1;

  strcpy(startDate, "");	
                        
  temp = strtok(buf," ");
  temp = strtok(NULL," ");
  strcat(startDate, temp); 
  temp = strtok(NULL," ");
  strcat(startDate, temp); 
  temp = strtok(NULL," ");
  if (debugOn == 1) printf("\n%s %s",startDate,temp);

  temp1 = strtok(temp,":");
  strcat(startDate, temp1); 
  time1 = atol(temp1) * 3600;

  temp1 = strtok(NULL,":");
  strcat(startDate, temp1); 
  time1 += atol(temp1) * 60;

  temp1 = strtok(NULL,":");
  time1 += atol(temp1);
                        
  return time1;

}

/* This function returns the number of processes in system by calling psCmd
 *	char *psCmd = "ps -ef | wc"; 
 */

int FindProcNum()
{
  int procNum = 0;
  char *psCmd = "ps -ef | wc";
  char buf[BUF_SIZE];
  char *str;
  FILE *ptr;
             
  if ((ptr = popen(psCmd, "r")) != NULL)
  {
    while (fgets(buf, BUF_SIZE, ptr) != NULL)
    {   
      if (debugOn == 1) printf("\n%s", buf); 
      str = strtok(buf," ");
      procNum = atoi(str);
    }

    pclose(ptr);
  }

  return(procNum);

}

/* This function returns the M_utilization by calling vsCmd + time interval
 *	char *vmstatCmd = "vmstat ";
 *  Modified to return available system capacity, that 100 - utilization 
 */

int 
FindUtil(int timeInterval)
{
  FILE *ptr;
  char *vmstatCmd = "vmstat ";
  char tmpCmd[30], *str;
  char buf[BUF_SIZE];
  int uUser, uSys, numIter;
  int jumpNum;

#ifdef SunOS 
  jumpNum = 19;
#else
  jumpNum = 13;
#endif

  sprintf(tmpCmd, "%s %d %d", vmstatCmd, timeInterval, 2);
  if ((ptr = popen(tmpCmd, "r")) != NULL)
  {
    while (fgets(buf, BUF_SIZE, ptr) != NULL)
    {
      str = fgets(buf, BUF_SIZE, ptr);
      str = fgets(buf, BUF_SIZE, ptr);
      str = fgets(buf, BUF_SIZE, ptr);
      str = fgets(buf, BUF_SIZE, ptr);
      str = fgets(buf, BUF_SIZE, ptr);

      if (debugOn == 1) printf("\nNew Cycle%s", buf);

      str = strtok(buf,"  ");
      for(numIter = 0; numIter < jumpNum; numIter++)
        str = strtok(NULL,"  ");

      uUser = stoi(str);
      str = strtok(NULL,"  ");
      uSys = stoi(str);

      if (debugOn == 1) printf("\nThe Utilization percentage is %d",(uSys + uUser));

    }
    pclose(ptr);
	
  }
  
  return(uSys + uUser);

}

/* This function calculates the mean value of the lastest elements passed
 * by the parameter *values. The number of elements is also passed in 
 * parameter num_elements 
 * 
 * This method needs to be modified, it should be able to choose randomaly
 * elements to calculate the mean value
 */

double 
CalculateAver(double *values, int maxnum, int numElements)
{
  int i = 0, elementIndex;
  double mean = 0.0, sd = 0.0, temp = 0.0, var = 0.0;
        
  for (i = 0; i < numElements; i++)
  {
    elementIndex = (rand() * (maxnum-1) /32767);
    mean += values[elementIndex];
  }
  mean /= numElements;

  return mean;

}

/* This function calculates the mean value of the J_lambda samples in
 * the last hour.
 * parameter hour_index gives the current hour's index
 */

double 
CalculateLAverage(int hourIndex)
{
  int counter = 0, i = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the lambdaValues is \n");

  for (i = 1; lambdaValues[hourIndex][i] != INVALID_LAMBDA; i++)
  {
    if (debugOn == 1) printf("%f ", lambdaValues[hourIndex][i]);
    sum += lambdaValues[hourIndex][i];
    counter++;
  }
  
  if (debugOn == 1) printf("\n the end of lambdaValues \n");
		
  if (counter != 0) 
  {
    return (sum / counter);
  } else return 0;

}

/* This function calculates the mean value of the J_lambda samples in
 * the last hour.
 * parameter hour_index gives the current hour's index
 */

double 
CalculateMAverage(int hourIndex)
{
  int counter = 0, i = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the muValues is \n");

  for (i = 1; muValues[hourIndex][i] != INVALID_MU; i++)
  {
    if (debugOn == 1) printf("%f ", muValues[hourIndex][i]);
    sum += muValues[hourIndex][i];
    if (muValues[hourIndex][i] != 0) counter++;
  }

  if (debugOn == 1) printf("\n the end of muValues \n");

  if (counter != 0) {
    return (sum / counter);
  } else return 0;

}

/* This function calculates the mean value of the J_lambda samples in
 * the last hour.
 * parameter hour_index gives the current hour's index
 */

double 
CalculateTAverage(int hourIndex)
{
  int counter = 0, i = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the thetaValues is \n");

  for (i = 1; thetaValues[hourIndex][i] != INVALID_THETA; i++)
  {
    if (debugOn == 1) printf("%f ", thetaValues[hourIndex][i]);
    sum += thetaValues[hourIndex][i];
    if (thetaValues[hourIndex][i] != 0) counter++;
  }
  
  if (debugOn == 1) printf("\n the end of thetaValues \n");

  if (counter != 0) {
    return (sum / counter);
  } else return 0;

}

/* This function calculates the mean value of the utilization samples in
 * the last hour.
 * parameter hour_index gives the current hour's index
 */

double 
CalculateUAverage(int hour_index)
{
  int counter = 0, i = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the rhoValues is \n");

  for (i = 1; rhoValues[hour_index][i] != INVALID_RHO; i++)
  {
    if (debugOn == 1) printf("%f ", rhoValues[hour_index][i]);
    sum += rhoValues[hour_index][i];
    counter++;
  }
  if (debugOn == 1) printf("\n the end of rhoValues \n");

  if (counter != 0) {
    return (sum / counter);
  } else return 0;

}

/* This function calculates the mean value of the J_lambda samples in 
 * the last hours. 
 * parameter hour_index gives the current hour's index
 * parameter hour_number shows the number of hours whose samples are going to
 * be calculated
 */

double 
CalculateAdaptiveLAverage(int hourIndex, int hourNumber)
{
  int counter = 0, i = 0, j = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the lambdaValues is \n");
		
  for (i = hourIndex; i >= (hourIndex-hourNumber+1); i--)
  {
    for (j = 1; lambdaValues[i][j] != INVALID_LAMBDA; j++)
    {
      if (debugOn == 1) printf("%f ", lambdaValues[i][j]);
      sum += lambdaValues[i][j];
      counter++;
    }
    printf("\n");
  }
  
  if (counter != 0) {
    return (sum / counter);
  } else return 0; 

}

/* This function calculates the mean value of the M_utilization samples in 
 * the last hours. 
 * parameter hourIndex gives the current hour's index
 * parameter hourNumber shows the number of hours whose samples are going to
 * be calculated
 */
double 
CalculateAdaptiveUAverage(int hourIndex, int hourNumber)
{
  int counter = 0, i = 0, j = 0;
  double sum = 0.0;

  if (debugOn == 1) printf("the rhoValues is \n");
  for (i = hourIndex; i >= (hourIndex-hourNumber+1); i--)
  {
    for (j = 1; rhoValues[i][j] != INVALID_RHO; j++)
    {
      if (debugOn == 1) printf("%f ", rhoValues[i][j]);
      sum += rhoValues[i][j];
      counter++;
    }
    
	if (debugOn == 1) printf("\n");
  }

  if (counter != 0) {
    return (sum / counter);
  } else return 0; 

}

/* This function calculates the std of the J_lambda samples in 
 * the last 24 hours, if have. 
 * parameter hour_index gives the current hour's index
 * parameter hour_number shows the number of hours whose samples are going to
 * be calculated
 */
double 
CalculateAdaptiveLSTD(int hourIndex)
{
  int counter = 0, i = 0, j = 0, k = 0, startingHour = 0;
  double sum = 0.0, temp;
  double average = 0.0;

  if (hourIndex >= 24) 
  {
    startingHour = hourIndex-24+1;
  }
  else 
  {
    startingHour = 0;
  }

  for (k = 0; k < (hourIndex-startingHour+1); k++) 
  {
    sum = 0.0;
    counter = 0;	
    lambdaMeanPredict[k] = CalculateAdaptiveLAverage(hourIndex, k+1);
    average = lambdaMeanPredict[k]; 
    for (i = hourIndex; i >= (hourIndex-k); i--)
    {
      for (j = 1; lambdaValues[i][j] != INVALID_LAMBDA; j++)
      {
        temp = lambdaValues[i][j] - average;
        temp = pow(temp,2.0);
        sum += temp;
        counter++;
      }
    }
    lambdaSTDPredict[k] = sqrt(sum/(counter-1)); 	
  }

  return (sqrt(sum / (counter - 1)));

}

/* This function calculates the std of the M_utilization samples in 
 * the last 24 hours, if have. 
 * parameter hourIndex gives the current hour's index
 * parameter hour_number shows the number of hours whose samples are going to
 * be calculated
 */

double 
CalculateAdaptiveUSTD(int hourIndex)
{
  int counter = 0, i = 0, j = 0, k = 0, startingHour = 0;
  double sum = 0.0, temp;
  double average = 0.0;

  if (hourIndex >= 24) 
  {
    startingHour = hourIndex-24+1;
  }
  else 
  {
    startingHour = 0;
  }

  for (k = 0; k < (hourIndex-startingHour+1); k++) 
  {
    sum = 0.0;
    counter = 0;	
    muMeanPredict[k] = CalculateAdaptiveUAverage(hourIndex, k+1);
    average = muMeanPredict[k]; 
    for (i = hourIndex; i >= (hourIndex-k); i--)
    {
      for (j = 1; rhoValues[i][j] != INVALID_RHO; j++)
      {
        temp = rhoValues[i][j] - average;
        temp = pow(temp,2.0);
        sum += temp;
        counter++;
      }
    }
    muSTDPredict[k] = sqrt(sum / (counter - 1)); 	
  }

  return (sqrt(sum / (counter - 1)));

}

/* set initial value for J_lambda and M_utilization
 */
void
InitValues() {
  int i, j;

  for(i = 0; i < MAX_HOUR; i++) 
  {
    for (j = 0; j < MAX_SAMPLE_SIZE; j++)
    { 	
      lambdaValues[i][j] = INVALID_LAMBDA;
      muValues[i][j] = INVALID_MU;
      thetaValues[i][j] = INVALID_THETA;
      rhoValues[i][j] = INVALID_RHO;
    }
  }

}

/* This function calculates the mean value of the M_utilization samples in 
 * the last hours. 
 * parameter hour_index gives the current hour's index
 * parameter hour_number shows the number of hours whose samples are going to
 * be calculated
 */
void 
RecordData(int hourIndex)
{
  FILE *fdData;
  char dataFileName[20];
  char dataLine[80];
  double servRate;
  int i = 0, j = 0;

  sprintf(dataFileName, "%s%s%d", startDate, data_log, hourIndex/24); 
  if ( (fdData = fopen(dataFileName,"w") )==NULL )
  {	
    if (debugOn == 1) printf("\ncannot open data log file to write");
    exit(1);
  }
			
  fputs("Utilization Lambda Servicerate\n", fdData);

  for (i = (hourIndex-23); i <= hourIndex; i++)
  {
    for (j = 1; rhoValues[i][j] != INVALID_RHO; j++)
    {
      if (debugOn == 1) printf("%f ", rhoValues[i][j], lambdaValues[i][j]);
      if (rhoValues[i][j] == 0)
      {
        servRate = 0;
      }
      else
      {
        servRate = lambdaValues[i][j] / rhoValues[i][j];
      }
      sprintf(dataLine, "%f %f %f \n", rhoValues[i][j], lambdaValues[i][j], 
        servRate);
      fputs(dataLine, fdData);
    }
    fputs("*****next hour start***** \n", fdData);
  }
  
  fclose(fdData);

}

/* Read the parameters from the configuration file */

void 
ReadConfig()
{
  FILE *fdConfig;
  char bufConfig[100];
  char ghsConfig[100];
  char *temp, *temp1;
  char hostName[80];
  int i = 0, j = 0;

  sprintf(ghsConfig, "%s%s", getenv("GHS_HOME"), "/ghs_config.dat");
  if ( (fdConfig = fopen(ghsConfig, "r") ) != NULL )
  {	
    while (fgets(bufConfig, 100, fdConfig) != NULL)
    {   
      if ((bufConfig[0] == 47)&&(bufConfig[1] == 47))
      { /*Process the comments here */
      }
      else {
        strcpy(configArray[i],bufConfig);
        i++; // number of configuration line
      }
    }
  }
  else {
    if (debugOn == 1) printf("\n ghsHome is %s \n", getenv("GHS_HOME"));
    if (debugOn == 1) printf("\nCannot open the configuration file");
    exit(1);
  }

  if (debugOn == 1) printf("\nReading configuration parameters... done");
  if (debugOn == 1) printf("\nPrinting configuration parameters");

  for(j = 0; j < i; j++)
  {	
    temp = strtok(configArray[j]," ");
    temp1 = strtok(NULL," ");

    if (strcmp(temp, "UNIT_TIME") == 0) // unit_time, usually it is an hour
    {	
      sampleTime = atoi(temp1);
      if (debugOn == 1) printf("\n The time to take one set of samples is: %d",sampleTime);
    }
		
    if (strcmp(temp, "INITIAL_SAMPLE_SIZE") == 0) // initial sample size
    {	
      numIterNext = atoi(temp1);
      if (debugOn == 1) printf("\n The initial sample size is: %d",numIterNext);
    }

    if (strcmp(temp, "LOOP_NUMBER") == 0) // the maximum measuring number of unit_time
    {	
      loopNum = atoi(temp1);
      if (debugOn == 1) printf("\n The loop number  is: %d",loopNum);
    }

    if (strcmp(temp, "MINIMUM_EFFECTIVE_LIFETIME") == 0) // unit_time, usually it is an hour
    {
      float pmTime;
      sscanf(temp1, "%f ", &pmTime);
      procMiniTime = pmTime;
      if (DEBUG_ON == 1) printf("\n the minimum effective process lifetime is : %f \n", procMiniTime);
    }

    if (strcmp(temp, "LOGFILE_NAME") == 0) // unit_time, usually it is an hour
    {
      //sscanf(temp1, "%s ", measureLog);
      char measureName[30];
      sscanf(temp1, "%s ", measureName);
      if (gethostname(hostName, 80) != 0)
      {
        printf("Can not get the host name to generate the measure log file \n");
        exit(0);
      }
      sprintf(measureLog, "%s%s", hostName, measureName); 
      if (DEBUG_ON == 1) printf("\n the log file's name is: %s \n", measureLog);
    }

    if (strcmp(temp, "LOGFILE_RENEW") == 0) // debug tag
    { 	
      logfileRenew = atoi(temp1);
      if (debugOn == 1) printf("\n The logfile Renew is set as: %d", logfileRenew);
    }

    if (strcmp(temp, "DEBUG") == 0) // debug tag
    { 	
      debugOn = atoi(temp1);
      if (debugOn == 1) printf("\n The debug set is: %d",debugOn);
    }

  }

  fclose(fdConfig);
}

/* void 
ProduceTimeSeries()
{
  int num_i = 0, time_inter = 0;

  num_i = 1;
  time_inter = sampleTime / numIter;
  while(num_i < numIter)
  {
    a[num_i] = num_i * time_inter; 
    num_i++;
  }
  a[num_i] = sampleTime;

  for(num_i = 0; num_i < numIter; num_i++)
  {
    if (debugOn == 1) printf(" %d",a[num_i]);
    if ((num_i+1) % 10 == 0)
    if (debugOn == 1) printf("\n");
  }

} */

/* get the number of finished processes after the specific previous process whose 
 * information is given by the firstline of the output of lastcomm utility on last call
 */

struct _JobStatistics 
GetFinishedProcNum(char *firstLine, float timePeriod)
{
  int iLc = 0, jLc = 0; // finished process number
  int flLength = 0, tempLc = 0;
  char *str, timeStr[20];
  char bufNext[100], bufTmp[100];
  char tokensFirst[20][20];
  FILE *ptr;
  // add later 
  double averServ, averSTD;
  struct _JobStatistics tmpJobstat;
  int matchedNum; //matched number of index in a line
  float procTime, arrivalRate;
  int jobNum; //the number of jobs which service time is satisfied with the constraint
  int i;

  for (i = 0; i < MAX_PROC_NUM; i++) jobServTime[i] = 0;
  jobNum = 0;

  /* Tokenize the first string */

  str = strtok(firstLine," ");
  strcpy(tokensFirst[0],str);
  if (debugOn == 1) printf("\n the first string\n\n");
  if (debugOn == 1) printf("\n%s",str);

  iLc=1;
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); iLc++;
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);
  str=strtok(NULL," "); strcpy(tokensFirst[iLc++] , str);
  if (debugOn == 1) printf("\n%s",str);

  flLength = iLc;
  if (debugOn == 1) printf("\nThe length of the first line is %d \n", flLength);
  iLc= 0;

  /* find current lastcommand output and compare the first line of it
   * with tokensFirst which store the first line of the last lastcommand
   * output
  */

  if ((ptr = popen("lastcomm", "r")) != NULL)
  {
    while ((fgets(bufNext, BUF_SIZE, ptr) != NULL) && (iLc < MAX_PROC_NUM) )
    {   
      if (debugOn == 1) printf("bufNext=  %s",bufNext);
      strcpy(bufTmp, bufNext);
      str = strtok(bufNext," ");
      strcpy(timeStr, str);

      jLc = 0;
      matchedNum = 0;

      if (strncmp(str,tokensFirst[jLc],strlen(str)) == 0) matchedNum++;   
      if (debugOn == 1) printf("\nYou are in the loop");
      jLc = 1;
      tempLc = 0;
      
      while( jLc < flLength )
      {
        if (debugOn == 1) printf("\nYou are in the inner loop");
        if (jLc == 2) 
        {
          str = strtok(NULL," ");
          jLc++;
          matchedNum++;
        }
								
        str = strtok(NULL," ");
        if (debugOn == 1) printf("\njLc=%d %s %s", jLc, str, tokensFirst[jLc]);

        if (strncmp(str, "secs", strlen(str)) == 0)
        {
          sscanf(timeStr,"%f",&procTime);
          if ( procTime >= procMiniTime)
          {
            jobServTime[jobNum] = procTime;
            if (debugOn == 1) printf("the job %d service time is %f \n", jobNum, jobServTime[jobNum]);
            //fputs(bufTmp, ptrtmp);
            //fflush(ptrtmp);
            jobNum++;
          }
        }
						
        if (strncmp(str,tokensFirst[jLc], strlen(str)) == 0) matchedNum++;
        jLc++;
        strcpy(timeStr, str);
      };

      if ( matchedNum == flLength)
      {
        if (debugOn == 1) printf("the line is mapped \n");
        // the processes should not include the first line
        if (procTime >= procMiniTime) jobNum--;
        break;
      }
      else iLc++;
      //if (debugOn == 1) printf("\niLc=%d\n", iLc);
    }//End of while

    if (debugOn == 1) printf("The number of commands executed in this interval are: %d\n\n ", jobNum);
  }

  pclose(ptr);

  // *************************** error on job arrival calculation *******************
  if (jobNum > 0 )
  {
    arrivalRate = jobNum / timePeriod;
    averServ = CalculateMean(&jobServTime[0], jobNum);
    averSTD = CalculateSTD(&jobServTime[0], jobNum);
  } 
  else {
    arrivalRate = 0;
    averServ = 0;
    averSTD = 0;
  }
	
  tmpJobstat.arrivalRate = arrivalRate;
  tmpJobstat.serviceRate = averServ;
  tmpJobstat.serviceSTD = averSTD;

  //if ( (matchedNum != flLength) || (jobNum== 0) ) tmpJobstat.arrivalRate = -1;

  return(tmpJobstat);
}

/* 
 * main program
 */

int 
main()
{
  /* These are the commands that are used for various measurements */

  char *lastCmd = "lastcomm | wc";       
  char *dateCmd = "date";
  char vmTime[20]; 
  int preNum;
             
        
  /* These are the times in seconds(measured from the parsed output of the date command)
   * and converted into seconds) that are measured at the beginning and the ending of 
   * one hour. The difference gives the physical time(world time) elapsed to take one set of
   * readings
  */

  long time1 = 0, time2 = 0;

  /* These are some temporary variables and some flags
  that are used in the program */

  int i = 0, j = 0, i_lc = 0, j_lc = 0, length = 0, fl_length = 0, temp_lc = 0;
  char *str, *temp, *temp1, bufLastCMD[BUF_SIZE];
        
  int oldPs = 0, currentPs = 0;   
  int cTime = 0, sTime = 10, mTime = 10;
  int jobArrival = 0;
  int p = 0, num_k = 0;

  /* initial unit_time for measuring, the maximum measuring time_unit */
  int loopCtr = 0;
  int hourIndex = 0;	 

  float values[10000];
 
  double mean = 0.0, sd = 0.0, lambda = 0.0, miu = 0.0, therta = 0.0;
  double lambdaAdaptAvg = 0.0, lambdaAdaptSTD;
  double rhoAdaptAvg = 0.0, rhoAdaptSTD;
  char buf[BUF_SIZE], timebuf[BUF_SIZE], firstline[BUF_SIZE];
  char newFileTag[10];
  FILE *ptr;
  time_t currTime = 0;
  int measureTime = 0;
                		
  //FILE *fd_config; // config file pointer

  time_t currentHour = 0;
  struct timeval tv;

  ReadConfig();

  gettimeofday(&tv, NULL);
  time1 = tv.tv_sec;
  if (debugOn == 1) printf("\n The time in seconds is %d\n\n",time1);
  fflush(stdout);
	

  /* Open the measure.log file to store system infomation */
  int newFileSign = 0;

  if ( (fdLog = fopen(measureLog,"r") ) == NULL )
  {
    if (debugOn == 1) printf("\n the log file doesn't exist, create a new measure log file \n");
    newFileSign = 1;
  }
  else 
  {
    fclose(fdLog);
    //printf("\n the log file %s is existing, do you want to add more measurement into this file (Y/N): ", measureLog);
    //scanf("%s", newFileTag);
    //if (strcmp(newFileTag, "Y") == 0) newFileSign = 0; 
    if (logfileRenew == 0) newFileSign = 0; 
    else newFileSign = 1;
  }

  if (newFileSign == 0) fdLog = fopen(measureLog, "r");
  else fdLog = fopen(measureLog, "w");

  if (fdLog == NULL) 
  {
      printf("\n can not create the measurement log file \n");
      exit(1);
  }  

  if (newFileSign == 1) 
  {
    fputs("date meanUtil meanLambda meanServ meanStd\n", fdLog);
    fflush(fdLog);
    lineNum = 1;
  } 
  else
  {
    lineNum = GetLineNum(measureLog);
    currTime = time1 / 3600;
    if (debugOn == 1) printf("the currTime is %d \n", currTime);

    while (fgets(buf, 100, fdLog) != NULL)
    {
      str = strtok(buf, " ");
      sscanf(str, "%d", &measureTime);
      if (debugOn == 1) printf("the time is %d \n", measureTime);
    }

    if (measureTime == 0) measureTime = currTime; 
    
    fclose(fdLog);

    if ( (fdLog = fopen(measureLog, "a")) == NULL)
    {
      printf("\n can not open the measurement log file \n");
      exit(1);
    }  

    float invalidValue = INVALID_PARA_VALUE;
    if ( (currTime - measureTime) > MAX_HOUR )
    {
      fclose(fdLog);
      RemoveLines(measureLog, lineNum - 1);
      fdLog = fopen(measureLog, "a");
    }
    else
    {
      while ( measureTime < currTime) 
      {
        if ( lineNum >= (MAX_HOUR + 1) )
        {
          fclose(fdLog);
          RemoveLines(measureLog, (lineNum - MAX_HOUR));
          lineNum = MAX_HOUR;
          fdLog = fopen(measureLog, "a");
        }

        sprintf(currHour, "%d ", measureTime);
        sprintf(meanLambda, "%f ", invalidValue);
        sprintf(meanMiu, "%f ", invalidValue);
        sprintf(meanUtilization, "%f ", invalidValue);
        sprintf(meanTherta, "%f \n", invalidValue);
  
        fputs(currHour, fdLog);
        fputs(meanUtilization, fdLog);
        fputs(meanLambda, fdLog);
        fputs(meanMiu, fdLog);
        fputs(meanTherta, fdLog);
        measureTime ++;
        lineNum ++;
      }
    }
  }
  
  /* if ( (ptrtmp = fopen("wmwm.txt","w") ) == NULL )
  {
    if (debugOn == 1) printf("\ncannot open log file to write");
    exit(1);
  } */

  /*Initialize the pseudo random number generator */

  InitValues();
  srand(32767);

  /*loop until the maximum measuring number is reached */

  while (loopCtr < loopNum)
  {

    numIter = numIterNext; 
    //numIterNext= 0;
    gettimeofday(&tv, NULL);
    currentHour = tv.tv_sec / 3600 ;

    /* initialize the time interval array*/ 

    for(i = 0; i < numIter; i++)
    {	
      values[i] = 0;
      a[i] = 0;
    }
                
    /* measure system parameters in a time_unit */

    i = 1;
    currSampleNum = 1;

    measureInterval = 3600 / numIter;

    while (i < numIter+1)
    { 

    /* measure the number of processes executed in the system, until this point from the out
    put of the command "lastcomm | wc". Note this information is maitained in a file called 
    /var/adm/pacct recent values overwrite the old values as the file becomes very large. */ 

      mTime = 20; // measurement period is fixed as 20 second

      cTime += mTime;
				
      if (debugOn == 1) printf("\n measure time is :%d", mTime);

      lastcommOn = 0;
      while ( lastcommOn == 0) 
      {
        if ((ptr = popen("lastcomm", "r")) != NULL)
        {
          if (fgets(bufLastCMD, BUF_SIZE, ptr) != NULL)
          {   
            strcpy(firstline, bufLastCMD);
            if (debugOn == 1) printf("\nThe first line is: %s",firstline);
            lastcommOn = 1;
          }
          else 
          {
            if (debugOn == 1) printf("the lastcomm can't be executed\n");
            sleep(1);
          }
          //if (debugOn == 1) printf("%s", firstline);
          pclose(ptr);
        }
      }

      //oldPs = FindProcNum();
      rhoValues[hourIndex][currSampleNum] = FindUtil(mTime) * 1.0/100.0;

      if (debugOn == 1) printf("After %d seconds ",mTime);
      fflush(stdout);
      //currentPs = FindProcNum(); 

      sampleStatistics = GetFinishedProcNum(firstline, mTime);

      lambda = sampleStatistics.arrivalRate;
      miu = sampleStatistics.serviceRate; // miu is actually the average job service time here
      therta = sampleStatistics.serviceSTD;

      /* if (lambda == -1) 
			{
				//numIter--;
				printf("no job found in the last measurement interval \n");
				i++;
				continue;
			} */
			/*
			if (lambda == -1) 
			{
				miu = 0;
				therta = 0;
				lambda = 0;
			} else
			{
			miu = sampleStatistics.serviceRate; // miu is actually the average job service time here
			therta = sampleStatistics.serviceSTD;
			} */
			
			//jobArrival = (currentPs-oldPs) + i_lc;
			//if (debugOn == 1) printf("\nJ_arrival : %d",jobArrival);
			//lambda = (double)jobArrival/(double)mTime;

      if (debugOn == 1) printf("\nJ_arrival : %f",lambda);
      if (debugOn == 1) printf("\nJ_serviceRate : %f",miu);
      if (debugOn == 1) printf("\nJ_serviceSTD : %f",therta);

      values[i] = lambda;

      lambdaValues[hourIndex][currSampleNum] = lambda;
      muValues[hourIndex][currSampleNum] = miu;
      thetaValues[hourIndex][currSampleNum] = therta;

      currSampleNum++;

      /* This is the no measure zone in which the process just sleeps */
      if (debugOn == 1) printf("\n\nNo measure zone starts\n");	
      //if (debugOn == 1) printf("a[i]=%d a[i-1]= %d i=%d",a[i],a[i-1],i);
      //sTime = a[i]-a[i-1]-mTime;

      if (i == numIter) 
      {
        sTime = 3600 - mTime - (numIter-1)*measureInterval;
      } 
      else sTime = measureInterval - mTime;

      if (debugOn == 1) printf("\nsTime=%d", sTime);
      if (sTime > 0) sleep(sTime);                 
					
      if (debugOn == 1) printf("\n\nNo measure zone ends\n");
		
      if (debugOn == 1) printf("\n\n");

      i++;

    } // end of measurement during one unit_time for loop

    if (debugOn == 1) printf("Calculate Sampe size for next loop and write the meausrement to datafile\n\n");
    fflush(stdout);

    gettimeofday(&tv, NULL);
    time2 = tv.tv_sec;
    if (debugOn == 1) printf("\nThe time in seconds is %d",time2);
    if (debugOn == 1) printf("\nThe measured time difference is %d\n\n",(time2-time1));

    //if (debugOn == 1) printf("The adaptive average over the last 24 unit_times is %f", lambdaAdaptAvg);
    sprintf(currHour, "%d ", currentHour);
    sprintf(meanLambda, "%f ", CalculateLAverage(hourIndex));
    sprintf(meanMiu, "%f ", CalculateMAverage(hourIndex));
    sprintf(meanUtilization, "%f ", CalculateUAverage(hourIndex));
    sprintf(meanTherta, "%f \n", CalculateTAverage(hourIndex));

    if (debugOn == 1) printf("Start writing the meausrement to datafile...\n\n");
    fflush(stdout);

    fputs(currHour, fdLog);
    fputs(meanUtilization, fdLog);
    fputs(meanLambda, fdLog);
    fputs(meanMiu, fdLog);
    fputs(meanTherta, fdLog);
		
    /* sd is calculated for all the samples obtained in the lasthour */

    //sd = CalculateSTD(values,numIter);
    lambdaAdaptSTD = CalculateAdaptiveLSTD(hourIndex);
    rhoAdaptSTD = CalculateAdaptiveUSTD(hourIndex);

    fflush(fdLog);
    hourIndex++;

    if (hourIndex < 24) preNum = hourIndex;
    else preNum = 24;

    if (muMeanPredict[preNum-1] == 0) 
    {
      if (debugOn == 1) printf("the utilization prediction is 0%, so the number of iterations remains %d", numIterNext);
    }
    else 
    {
      numIterNext = ceil( (1536.64 / 24) * pow((muSTDPredict[preNum-1] / muMeanPredict[preNum-1]), 2.0));
    }

    if (numIterNext > MAX_SAMPLE_SIZE) 
    {
      numIterNext = MAX_SAMPLE_SIZE;
    }
    else if (numIterNext < MIN_SAMPLE_SIZE)
    {
      numIterNext = MIN_SAMPLE_SIZE;
    }

    if (debugOn == 1) printf("\nThe number of iterations that should be in the next observation is %d", numIterNext);

    loopCtr ++;
                          
  } /*outside while loop */
  
  fclose(fdLog);
  //fclose(ptrtmp);
	
  return 0;

}
